/*
 * This file contains declarations for the low level routines for the
 * HANDYSCOPE
 */
#include <dos.h>

typedef unsigned char byte;
typedef unsigned int  word;


#define LPT1 0x278
#define LPT2 0x378
#define LPT3 0x3bc

#define NRSENSITIVITIES 6
#define INPUTRANGE_05   0   //  0.5 Volt full scale
#define INPUTRANGE_1    1   //  1   Volt full scale
#define INPUTRANGE_2    2   //  2   Volt full scale
#define INPUTRANGE_5    3   //  5   Volt full scale
#define INPUTRANGE_10   4   // 10   Volt full scale
#define INPUTRANGE_20   5   // 20   Volt full scale

#define NRCHANNELS      2
#define CH1             0
#define CH2             1

#define NRSAMPLES       100 // number of samples supported in this demo


/*
 * the address of the printer port the HANDYSCOPE is connected to
 */
 extern word wADaddress;

/*
 *  the array tha data is stored in
 */
 extern word wMeasurementArray[NRCHANNELS][NRSAMPLES];

/*
 * an array with the HANDYSCOPE sensitivities, used to calculate the
 * measured value in volts
 */
 extern float rInputValue[NRSENSITIVITIES];



extern void SetSensitivity( byte byInputRangeCh1 , byte byInputRangeCh2 );
/* DESCRIPTION
 *   This routine sets the sensitivity for both channels of the HANDYSCOPE
 * INPUT
 *   byte byInputRangeCh1   pointer to the sensitivity array element for Ch1
 *   byte byInputRangeCh2   pointer to the sensitivity array element for Ch2
 * OUTPUT
 *   -
 */


extern word MeasureOneSampleBinary( byte byCh, word wMeasureDelay );
/* DESCRIPTION
 *   This routine measures one sample with the HANDYSCOPE and uses a delay
 *   before the sample. The data is returned in 12 bits binary
 * INPUT
 *   byte  byCh            the channel te measure from
 *   word  wMeasureDelay   the delay to use. This delay is NOT defined in
 *                         real time, but a loop counter. To create "real"
 *                         times, at start of the program the speed of the
 *                         computer has to be determined.
 * OUTPUT
 *   return value          the measured value,    0 = max. neg. full scale
 *                                             4096 = max. pos. full scale
 */



extern float MeasureOneSampleInVolts( byte byCh,
                                      word wMeasureDelay,
                                      float rSens );
/* DESCRIPTION
 *   This routine measures one sample with the HANDYSCOPE and uses a delay
 *   before the sample. The data is returned in volts.
 * INPUT
 *   byte  byCh            the channel te measure from
 *   word  wMeasureDelay   the delay to use. This delay is NOT defined in
 *                         real time, but a loop counter. To create "real"
 *                         times, at start of the program the speed of the
 *                         computer has to be determined.
 *   float rSens           The full scale sensitivity of the channel in volts
 * OUTPUT
 *   return value          the measured value in volts
 */


extern void MeasureHandyScope( int nMeasureCount, word wMeasureDelay );
/* DESCRIPTION
 *   This routine measures a number of samples with the HANDYSCOPE
 *   and uses a delay for each sample. The data is stored binary.
 * INPUT
 *   int  nMeasureCount    the number of samples to be measured
 *   word wMeasureDelay    the delay to use. This delay is NOT defined in
 *                         real time, but a loop counter. To create "real"
 *                         times, at start of the program the speed of the
 *                         computer has to be determined.
 * OUTPUT
 *   the sampled data is stored in a global array called wMeasurementArray
 */
