/*
 * File Types.h
 */

#ifndef __TYPES_H
#define __TYPES_H

/*
 * System include files
 */
#include <dos.h>
#include <_defs.h>
#include <_nfile.h>

/*
 * Custom include files
 */
#include "const.h"

typedef enum BOOL { FALSE,    // 0
                    TRUE,     // 1
                    LAST_BOOL // 2
                  };
typedef unsigned char BYTE;
typedef unsigned int UINT;
typedef UINT WORD;
typedef unsigned long DWORD;

typedef enum OPERATE{ FREEZE,      // 0
                      OPLIVE,      // 1
                      LAST_OPERATE // 2
                    };

typedef enum SLOPE{ POS,       // 0
                    NEG,       // 1
                    LAST_SLOPE // 2
                  };

typedef enum TRIGSOURCE{ CH1,        // 0
                         CH2,        // 1
                         BOTH,       // 2
                         CH1ANDCH2,  // 3
                         CH1ORCH2,   // 4
                         EXT,        // 5
                         NO_TRIG,    // 6
                         LAST_TRIGCH // 7
                       };

typedef enum SIGTYPE{ AC,          // 0
                      DC,          // 1
                      LAST_SIGTYPE // 2
                    };

typedef enum ARROWTYPE{ TRIG1,         // 0
                        OFFSET1,       // 1
                        TRIG2,         // 2
                        OFFSET2,       // 3
                        LAST_ARROWTYPE // 4
                      };

typedef enum DEVICETYPE{ LIVE,            // 0
                         HOLD,            // 1
                         FFT,             // 2
                         VOLT,            // 3
                         TRANS,           // 4
                         LAST_DEVICE_TYPE // 5
                       };

typedef enum MEASMODE{ CHAN1,        // 0
                       CHAN2,        // 1
                       ALT,          // 2
                       CHOP,         // 3
                       ADD,          // 4
                       COMPARE,      // 5
                       XYPLOT,       // 6
                       LAST_MEASMODE // 7
                     };

#define LOBYTE(w)      ((BYTE)(w))
#define HIBYTE(w)      ((BYTE)((UINT)(w) >> 8))

typedef struct ZOOMSAMPRANGE
{
 long lBeginSampVal;
 long lEndSampVal;
} _ZOOMSAMPRANGE;



/* Measurement-related variables */
typedef struct HS508_MEASDEP
{
 float      SampleFreq;
 long       NumPostSamples;
 long       NumPreSamples;
 BYTE       TrigLevel;
 BYTE       TrigHysteresis;
 SLOPE      TrigSlope;
 TRIGSOURCE TrigSource;
 MEASMODE   MeasurementMode;

} _HS508_MEASDEP;


typedef struct MEASDEP
{
  BYTE       Switch[NUMCHANNELS];  // holds all switch settings (bitwise)
  float      SampleFreq;           // holds the sample frequency
  long       NumPostSamples;       // holds the number of post trigger samples
  long       NumPreSamples;        // holds the number of pre trigger samples
  BYTE       TrigLevel;            // holds the triggerlevel
  BYTE       TrigHysteresis;       // holds the trigger hysteresis
  float      DCLevel[NUMCHANNELS]; // holds the DC levels (hardware offsets)
  BOOL       ACDC[NUMCHANNELS];    // holds the coupling settings
  SLOPE      TrigSlope;            // holds the trigger slope
  TRIGSOURCE TrigSource;           // holds the trigger source
  long int   TimeOut;              // holds the trigger timeout

} MEASDEP;


typedef struct EQUIP_OVERALL_INFO
{
  MEASMODE MeasurementMode; // indicates which channels to display
  OPERATE  OperationMode;    // and how to display them

}_EQUIP_OVERALL_INFO;

#endif //#ifndef Types.h
