/* File Ad_Tp112.c */

/* System include files */

extern "C" {
 #include <dos.h>
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
 #include <string.h>
 #include <stdio.h>
}

/* Custom include files */
#include "ad_tp112.h"
#include "addmath.h"
#include "globvars.h"
#include "types.h"
#include "const.h"


/* Global Variable Declarations */
// Following buffers specify where to find the CONTROL and DATA registers on the I/O map */
BYTE ContReg[] = {0, 4, 6};
BYTE DataReg[] = {1, 5, 7};

BYTE Level1_Con_Ad[]     = {0, 0};
BYTE Level2_Con_Ad[]     = {1, 1};
BYTE Trig_Niv_Con_Ad[]   = {2, 2};
BYTE Pga_Con_Ad[]        = {3, 3};
BYTE Data_Nul_Con_Ad[]   = {4, 4};
BYTE Irq_Con_Ad[]        = {5, 5};
BYTE Trig_Reset_Con_Ad[] = {6, 6};

/* Variables for the DATA register ---> are being initialized */
BYTE Post_Lo_Reg        = 0;   // register 0
BYTE Post_Hi_Reg        = 0;   // register 1
BYTE Output_Reg         = 0;   // register 2
BYTE TimeBase_Reg       = 0;   // register 3
BYTE Mode_Reg           = 0;   // register 4
BYTE Reset_Reg          = 0;   // register 5
BYTE Count_Reg          = 0;   // register 6
BYTE Write_TimeBase_Reg = 0;   // register 7

/* Register Contants Variables */
BYTE Level1_Reg_Ad[]     = {0, 0};
BYTE Level2_Reg_Ad[]     = {0, 0};
BYTE Trig_Niv_Reg_Ad[]   = {0, 0};
BYTE Pga_Reg_Ad[]        = {0, 0};
BYTE Trig_Reset_Reg_Ad[] = {0, 0};

int Ad_PortAddress = 0x288;  /*!!!!!!!!!!!!!!!!!!!*/


BYTE Ch_Switch_Value[NUMCHANNELS][5] =
{
  {
    4,  //  1.25 V
    3,  //  2.50 V
    2,  //  5    V
    1,  // 10    V
    0   // 20    V
  },
  {
    4,  //  1.25 V
    3,  //  2.50 V
    2,  //  5    V
    1,  // 10    V
    0   // 20    V
  }
};



/*
 * Local function declarations
 */
int  Clip          (int nLo, int nValue, int nHi);
void SetPGA        (BYTE byCh, BOOL bAC_DC, BYTE bySetting);
BOOL ReadEOCBit    (void);
void SetPostCount  (WORD wPostCount);
void SetTrigLevel  (SLOPE Slope, BYTE byLevel, BYTE byHysteresis);
void SetTrigBit    (void);
void SetTimeBase   (float* fFreq);
void ClearSRAM     (void);
void GoToSRAMPos   (WORD wSRAMPos);
void ReadSRAMData  (TP112_MEASDEP* pMeasSettings);
void MeasureTP112  (TP112_MEASDEP* pMeasSettings);
void ReadDigitalInputs(TP112_MEASDEP* pMeasSettings);
void WriteDigitalOutputs(BYTE byValue);

/*
 * Ad_Tp112.c Local function definitions...
 */

int Clip( int nLo, int nValue, int nHi)
{
  if ( nValue > nHi )
  {
    nValue = nHi;
  }
  else
  {
    if ( nValue < nLo )
    {
      nValue = nLo;
    }
  }
  return nValue;
}



/* NAME:
 *   void SetPGA (BYTE byCh, BOOL bAC_DC, BYTE bySetting)
 * DESCRIPTION:
 *   This routine converts the switch data and writes it to the hardware
 * INPUT PARAMETERS:
 *   byCh        which chanel
 *   bAC_DC      coupling setting
 *   bySetting   the other switches (attenuators)
 * OUTPUT PARAMETERS:
 *   -
 */
void SetPGA (BYTE byCh, BOOL bAC_DC, BYTE bySetting)
{
 BYTE byPgaSetting;

 byPgaSetting = Ch_Switch_Value[byCh][bySetting];
 if (!bAC_DC)
 {
   byPgaSetting = byPgaSetting | 0x08;
 }
 Pga_Reg_Ad[byCh] = byPgaSetting;
 outportb( Ad_PortAddress+ContReg[byCh+1], Pga_Con_Ad[byCh] );
 outportb( Ad_PortAddress+DataReg[byCh+1], Pga_Reg_Ad[byCh] );
/*
 * Wait X msec for switching PGA
 */
 delay (PGA_SWITCHINGTIME);
} //SetPGA


/* NAME:
 *   BOOL ReadEOCBit( void )
 * DESCRIPTION:
 *   This routine reads the End Of Conversion bit
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   returnvalue is the End Of Conversion bit
 */
BOOL ReadEOCBit( void )
{
 return( BOOL(( inportb( int(Ad_PortAddress + 2) ) ) & 0x01 ));
} //ReadEOCBit


/* NAME:
 *   void SetPostCount (WORD wPostCount)
 * DESCRIPTION:
 *   This routine sets the value for the post counter
 * INPUT PARAMETERS:
 *   wPostCount  contains the new value for the post counter
 * OUTPUT PARAMETERS:
 *   -
 */
void SetPostCount (WORD wPostCount)
{
 /*
  * split the post counter value in lower and higher byte
  */
  Post_Lo_Reg = LOBYTE( wPostCount );
  Post_Hi_Reg = HIBYTE( wPostCount );
 /*
  * select the POST-LO register and store the lower byte of the value into it
  */
  outportb( Ad_PortAddress+ContReg[0], (unsigned char)POST_LO_CON );
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)Post_Lo_Reg );
 /*
  * select the POST-HI register and store the higher byte of the value into it
  */
  outportb( Ad_PortAddress+ContReg[0], (unsigned char)POST_HI_CON );
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)Post_Hi_Reg );
}//SetPostCount



/* NAME:
 *   void SetTrigLevel( SLOPE Slope, BYTE byLevel, BYTE byHysteresis )
 * DESCRIPTION:
 *   This routine sets the trigger level
 * INPUT PARAMETERS:
 *   Slope        the triggerslope
 *   byLevel      the value for the trigger level
 *   byHysteresis the trigger hysteresis
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTrigLevel( SLOPE Slope, BYTE byLevel, BYTE byHysteresis )
{
  int nHelp, nCh;

  for (nCh = 0; nCh <= 1; nCh++)
  {
    if (Slope == POS) // Positive Slope
    {
      nHelp = byLevel - byHysteresis;
      Level1_Reg_Ad[nCh] = Clip(0, nHelp, 255);
      Level2_Reg_Ad[nCh] = Clip(0, byLevel, 255);
      Trig_Niv_Reg_Ad[nCh] = 5; // 0000 0101
    }
    else // Negative Slope
    {
      nHelp = byLevel + byHysteresis;
      Level1_Reg_Ad[nCh] = Clip(0, nHelp, 255);
      Level2_Reg_Ad[nCh] = Clip(0, byLevel, 255);
      Trig_Niv_Reg_Ad[nCh] = 6; // 0000 0110
    }
    outportb( Ad_PortAddress+ContReg[nCh+1], Trig_Niv_Con_Ad[nCh] );
    outportb( Ad_PortAddress+DataReg[nCh+1], Trig_Niv_Reg_Ad[nCh] );

    outportb( Ad_PortAddress+ContReg[nCh+1], Level1_Con_Ad[nCh] );
    outportb( Ad_PortAddress+DataReg[nCh+1], Level1_Reg_Ad[nCh] );
    outportb( Ad_PortAddress+ContReg[nCh+1], Level2_Con_Ad[nCh] );
    outportb( Ad_PortAddress+DataReg[nCh+1], Level2_Reg_Ad[nCh] );
  }

} //SetTrigLevel


/* NAME:
 *   void SetTrigBit( void )
 * DESCRIPTION:
 *   This routine sets the trigger bit.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTrigBit ( void )
{
  int nHelp;

  nHelp = Mode_Reg;
  nHelp = nHelp & 63;
  nHelp = nHelp | 128; // Mode_Reg has to become: 10xx xxxx
  outportb( Ad_PortAddress+ContReg[0], MODE_CON );
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)nHelp );
} // SetTrigBit



/* NAME:
 *   void SetTimeBase (float* fFreq)
 * DESCRIPTION:
 *   This routine sets the timebase register. Some frequencies can be made
 *   directly by dividing the clock twice or four times. For other frequencies
 *   the 8254 has to be programmed with another value.
 * INPUT PARAMETERS:
 *   fFreq    the frequency to make
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTimeBase (float* fFreq)
{
 WORD wDivFactor;
 BYTE HighBytewDiv, LowBytewDiv;
 float fInputFreq;

 fInputFreq = 4e6; // this is the oscillator frequency
/*
 * a special division factor has to be calculated
 */
 if (*fFreq > 1e6)
 {
   *fFreq = 1e6;
 }
 if (*fFreq < 62)
 {
   fInputFreq = 4e4;
 }
 wDivFactor = (WORD)Round (double(fInputFreq/(*fFreq)));
 *fFreq = fInputFreq/wDivFactor;

/*
 * Initialize timer IC 8254, control word A1, A0=11
 */
 TimeBase_Reg = 3;
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)TimeBase_Reg);
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)54); // 00 11 011 0

/*
 *  00 [=select counter 0]
 *  11 [=write LSBYTE first]
 * 011 [=mode 3, divide by n counter]
 *   0 [=16-bits counter]
 */

/*
 * Load DivFactor
 */
 TimeBase_Reg = 0;
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)TimeBase_Reg);
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
 LowBytewDiv = LOBYTE(wDivFactor); // & 0x00FF;
 outportb( Ad_PortAddress + DataReg[0], (unsigned char) LowBytewDiv);
 HighBytewDiv = HIBYTE(wDivFactor); // & 0xFF00;
 outportb( Ad_PortAddress + DataReg[0], (unsigned char) HighBytewDiv);

 if (*fFreq < 62)
 {
 /*
  * Initialize timer IC 8254, control word A1,A0=11
  */
  TimeBase_Reg=3;
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)118);
 /*
  *  01 [=select counter 1]
  *  11 [=write LSBYTE first]
  * 011 [=mode 3, divide by n counter]
  *   0  [=16-bits counter]
  */

 /*
  * Load DivFactor
  */
  TimeBase_Reg = 1;
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)LOBYTE (100));
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)HIBYTE (100));
 }

/*
 * Select Clock Input
 */
 if (*fFreq < 62)
 {
  TimeBase_Reg = 8; // Select clck input b
 }
 else
 {
  TimeBase_Reg = 0; // Select clck input a
 }
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

} //SetTimeBase


/* NAME:
 *   void ClearSRAM (void)
 * DESCRIPTION:
 *   This routine clears the SRAM
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void ClearSRAM (void)
{
 int nCh;
 float fFreq;

 SetPostCount (32767L + 1);

 for (nCh = 0; nCh <= 1; nCh++)
 {
 /*
  * set Data zero register
  */
  outportb( Ad_PortAddress+ContReg[nCh+1], DATAZERO_CON );
  outportb( Ad_PortAddress+DataReg[nCh+1], (unsigned char)1); // 0000 0001
 }
/*
 * Set fastest TimeBase
 */
 fFreq = 1e6;
 SetTimeBase (&fFreq);
/*
 * Set Clck Source : Sample clock off
 *                   Clock to timebase
 *                   Data from ADC to SRAM and PC-bus
 *                   Conversion clock on
 *                   trigger direct
 */
 outportb( Ad_PortAddress+ContReg[0], MODE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)142);  // 1000 1110
/*
 * Reset Post Counter and Reset Address Counter
 */
 outportb( Ad_PortAddress+ContReg[0], RESET_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);    // 0000 0000
/*
 * Turn CLCK On
 */
 outportb( Ad_PortAddress+ContReg[0], MODE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)143);  // 1000 1111
} // ClearSRAM



/* NAME:
 *   void GoToSRAMPos (WORD wSRAMPos)
 * DESCRIPTION:
 *
 * INPUT PARAMETERS:
 *   wSRAMPos  to position in the SRAM to go to
 * OUTPUT PARAMETERS:
 *   -
 */
void GoToSRAMPos (WORD wSRAMPos)
{
  float fFreq;
  BYTE byHi, byLo;

 /*
  * Load postcounter with the address
  */
  byHi = (wSRAMPos & 0xFF00) >> 8;
  byLo =  wSRAMPos & 0x00FF;
  outportb( Ad_PortAddress+ContReg[0], POST_LO_CON );
  outportb( Ad_PortAddress+DataReg[0], byLo );
  outportb( Ad_PortAddress+ContReg[0], POST_HI_CON );
  outportb( Ad_PortAddress+DataReg[0], byHi );
 /*
  * Select fastest timebase
  */
  fFreq = 1e6;
  SetTimeBase (&fFreq);
 /*
  * Reset post counter
  */
  outportb( Ad_PortAddress+ContReg[0], RESET_CON);
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
 /*
  * Set Clck Source
  */
  outportb( Ad_PortAddress+ContReg[0],MODE_CON);
  outportb( Ad_PortAddress+DataReg[0],(unsigned char)130);/*129*/  // 1000 0001
 /*
  * Turn on Clck
  */
  outportb( Ad_PortAddress+ContReg[0],MODE_CON);
  outportb( Ad_PortAddress+DataReg[0],(unsigned char)131); // 1000 0011
  /*
  * Wait until TrigBit == '1'
  */
  delay(1);
  do
  {
   // just do nothing, wait
  }
  while (!ReadEOCBit());

 /*
  * Switch clock off
  */
  outportb( Ad_PortAddress+ContReg[0], MODE_CON);
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);

}  //GoToSRAMPos



/* NAME:
 *   void ReadSRAMData (TP112_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine reads the SRAM data to the data buffer in the PC RAM
 * INPUT PARAMETERS:
 *   pMeasSettings  contains measurement instrument settings
 * OUTPUT PARAMETERS:
 *   -
 */
void ReadSRAMData (TP112_MEASDEP* pMeasSettings)
{
 int nSampleIndex;
 long int lHelp, lCounter_Lo, lCounter;
 WORD wHelp;

/*
 * Switch clock off
 */
 outportb( Ad_PortAddress+ContReg[0], MODE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
/*
 * Read Address Counter
 */
 lCounter_Lo = (unsigned char)inportb (Ad_PortAddress + 0);
 lCounter = (long)inportb (Ad_PortAddress + 1)*256 + lCounter_Lo;
 lCounter = (lCounter & RAMSIZE_TP112);
 lHelp = lCounter - (pMeasSettings->NumPostSamples + pMeasSettings->NumPreSamples) - 5;
 if (lHelp < 0)
 {
   lHelp+=RAMSIZE_TP112;
 }
 GoToSRAMPos (lHelp);
/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress+ContReg[0], MODE_CON);
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)128);

 switch (pMeasSettings->MeasurementMode)
 {
  case CHAN1:
    for ( nSampleIndex = 0;
          nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
          nSampleIndex++
        )
    {
      *(pGlobalRAMDataCH1+nSampleIndex) = ( inportb(Ad_PortAddress+4)+
                                            256*(inportb(Ad_PortAddress+5)&0x0F)
                                          ) * MULPLYFACTOR_TP112;
     /*
      * Increment Address Counter
      */
      outportb( Ad_PortAddress+ContReg[0], COUNT_CON);
      outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
    }
    break;
  case CHAN2:
    for ( nSampleIndex = 0;
          nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
          nSampleIndex++
        )
    {
      *(pGlobalRAMDataCH2+nSampleIndex) = ( inportb(Ad_PortAddress+6)+
                                            256*(inportb(Ad_PortAddress+7)&0x0F)
                                          ) * MULPLYFACTOR_TP112;
     /*
      * Increment Address Counter
      */
      outportb( Ad_PortAddress+ContReg[0], COUNT_CON);
      outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
    }
    break;
  case ALT:
  case ADD:
  case CHOP:
  case XYPLOT:
    for ( nSampleIndex = 0;
          nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
          nSampleIndex++
         )
    {
      *(pGlobalRAMDataCH1+nSampleIndex) = ( inportb(Ad_PortAddress+4)+
                                            256*(inportb(Ad_PortAddress+5)&0x0F)
                                          ) * MULPLYFACTOR_TP112;
      *(pGlobalRAMDataCH2+nSampleIndex) = ( inportb(Ad_PortAddress+6)+
                                            256*(inportb(Ad_PortAddress+7)&0x0F)
                                          ) * MULPLYFACTOR_TP112;
     /*
      * Increment Address Counter
      */
      outportb( Ad_PortAddress+ContReg[0], COUNT_CON);
      outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
    }
    break;
 }

/*
 * Reset TrigChannel 1
 */
 outportb( Ad_PortAddress+ContReg[1], TRIGRESET_CON);
 outportb( Ad_PortAddress+DataReg[1], (unsigned char)0);
/*
 * Reset TrigChannel 2
 */
 outportb( Ad_PortAddress+ContReg[2], TRIGRESET_CON);
 outportb( Ad_PortAddress+DataReg[2], (unsigned char)0);

 ClearSRAM (); // For next measurement

} //ReadSRAMData



/* NAME:
 *   void MeasureTP112 (TP112_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine sets the TP112 card to perform one measurement on both
 *   channels
 * INPUT PARAMETERS:
 *   pMeasSettings   instrument and measurement settings
 * OUTPUT PARAMETERS:
 *   -
 */
void MeasureTP112 (TP112_MEASDEP* pMeasSettings)
{
 BYTE byHelp;

 SetTrigLevel( pMeasSettings->TrigSlope,
                pMeasSettings->TrigLevel,
	        pMeasSettings->TrigHysteresis
             );
 SetTimeBase( &(pMeasSettings->SampleFreq) );

 switch (pMeasSettings->TrigSource)
 {
   case CH1:     byHelp =   0;
                 break;
   case CH2:     byHelp =  64;
                 break;
   case NO_TRIG: byHelp = 128;
                 break;
   default:      byHelp = 192;
 } /* switch */

 SetPostCount( pMeasSettings->NumPostSamples );

 outportb( Ad_PortAddress+ContReg[1], DATAZERO_CON );
 outportb( Ad_PortAddress+DataReg[1], (unsigned char)0 ); // let ADC data pass
 outportb( Ad_PortAddress+ContReg[2], DATAZERO_CON );
 outportb( Ad_PortAddress+DataReg[2], (unsigned char)0 ); // let ADC data pass
/*
 * Reset Post Counter and Address Counter
 */
 outportb( Ad_PortAddress+ContReg[0], RESET_CON );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0 );
/*
 * Set Clck Source
 */
 Mode_Reg = (unsigned char)byHelp+14;
 outportb( Ad_PortAddress+ContReg[0], MODE_CON );
 outportb( Ad_PortAddress+DataReg[0], Mode_Reg );
/*
 * Reset Post Counter and Address Counter
 */
 outportb( Ad_PortAddress+ContReg[0], RESET_CON );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0 );
/*
 * Reset trig Ch1 and Ch2
 */
 outportb( Ad_PortAddress+ContReg[1], TRIGRESET_CON );
 outportb( Ad_PortAddress+DataReg[1], (unsigned char)0 );
 outportb( Ad_PortAddress+ContReg[2], TRIGRESET_CON );
 outportb( Ad_PortAddress+DataReg[2], (unsigned char)0 );
/*
 * Set Clck SRAM
 */
 Mode_Reg = (unsigned char)byHelp+15;
 outportb( Ad_PortAddress+ContReg[0], MODE_CON );
 outportb( Ad_PortAddress+DataReg[0], Mode_Reg );
} //MeasureTP112



/* NAME:
 *   void ReadDigitalInputs(TP112_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine reads the Digital inputs data to the data buffer
 *   in the PC RAM
 * INPUT PARAMETERS:
 *   pMeasSettings  contains measurement instrument settings
 * OUTPUT PARAMETERS:
 *   -
 */
void ReadDigitalInputs(TP112_MEASDEP* pMeasSettings)
{
  int nSampleIndex;
  long int lHelp, lCounter_Lo, lCounter;
  WORD wHelp;

 /*
  * Switch clock off
  */
  outportb( Ad_PortAddress+ContReg[0], MODE_CON);
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
 /*
  * Read Address Counter
  */
  lCounter_Lo = (unsigned char)inportb (Ad_PortAddress + 0);
  lCounter = (long)inportb (Ad_PortAddress + 1)*256 + lCounter_Lo;
  lCounter = (lCounter & RAMSIZE_TP112);
  lHelp = lCounter - (pMeasSettings->NumPostSamples + pMeasSettings->NumPreSamples) - 5;
  if (lHelp < 0)
  {
    lHelp+=RAMSIZE_TP112;
  }
  GoToSRAMPos (lHelp);
 /*
  * Set Clck Source
  */
  outportb( Ad_PortAddress+ContReg[0], MODE_CON);
  outportb( Ad_PortAddress+DataReg[0], (unsigned char)128);

  for ( nSampleIndex = 0;
        nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
        nSampleIndex++
      )
  {
    *(pGlobalRAMDataDig+nSampleIndex) = ((inportb(Ad_PortAddress+5)&0xF0) >> 4) +
                                         (inportb(Ad_PortAddress+7)&0xF0);
   /*
    * Increment Address Counter
    */
    outportb( Ad_PortAddress+ContReg[0], COUNT_CON);
    outportb( Ad_PortAddress+DataReg[0], (unsigned char)0);
  }

 /*
  * Reset TrigChannel 1 & 2
  */
  outportb( Ad_PortAddress+ContReg[1], TRIGRESET_CON);
  outportb( Ad_PortAddress+DataReg[1], (unsigned char)0);
  outportb( Ad_PortAddress+ContReg[2], TRIGRESET_CON);
  outportb( Ad_PortAddress+DataReg[2], (unsigned char)0);

  ClearSRAM (); // For next measurement

} //ReadSRAMData


void WriteDigitalOutputs( BYTE byValue )
/* DESCRIPTION
 *   This routine sets the digital outputs of the TP112
 * INPUT PARAMETERS
 *   byValue   The value to write
 * OUTPUT PARAMETERS
 *   -
 */
{
  outportb( Ad_PortAddress+ContReg[0], OUTPUT_CON);
  outportb( Ad_PortAddress+DataReg[0], byValue);
} //WriteDigitalOutputs

