/*
 * File SCOPE.C
 *
 * This program uses far pointers and therefore requires COMPACT, LARGE or
 * HUGE memory model to compile
 *
 *
 *
 * System include files
 */
 #include <graphics.h> // graphic display routines
 #include <stdio.h>    // standard io routines
 #include <alloc.h>    // memory allocation routines
 #include <conio.h>    // console io routines
 #include <stdlib.h>   // standard library with routines

/*
 * Custom include files
 */
 #include "globvars.h" // global variables
 #include "addmath.h"  // add some extra mathematic routines
 #include "types.h"    // the types usen in this program
 #include "const.h"    // the consts used in this program
 #include "measure.h"  // the measurement routines
/*
 * Local function declarations
 */
 void InitializeGlobalDataArrays (int nBeginRAMIndex);
 void InitializeLIVEInfo (EQUIP_OVERALL_INFO* pLIVEInfo);
 void SetScopeBoxDimensions (int* pHorzRes);
 void AllocateDataArrays (int nHorzRes);
 void FreeDataArrays ( void );


int main(void)
{
  MEASDEP            MeasDependencies; // structure with 'instrument settings'
  EQUIP_OVERALL_INFO LIVEInfo;         // structure with operating info
  int                nHorzRes;         // horizontal screen resolution

 /*
  * select VGA driver and hires mode that supports
  * multiple drawing colors.
  */
  int gdriver = VGA,
      gmode   = VGAHI,
      errorcode;

 /*
  * initialize graphics and local variables
  */
  initgraph(&gdriver, &gmode, "C:\\AP\\BC\\BGI");
 /*
  * read result of initialization
  */
  errorcode = graphresult();
  if (errorcode != grOk)
  {
   /*
    * an error occurred
    */
    printf( "Graphics error: %s\n", grapherrormsg( errorcode ) );
    printf( "Press any key to halt: . . ." );
    getch();
   /*
    * terminate with an error code
    */
    exit(1);
  }

  setbkcolor (DARKGRAY);
  cleardevice(); //clear screen

 /*
  * Perform several initialisations
  */
  SetScopeBoxDimensions (&nHorzRes);  // determine scope box dimensions
  AllocateDataArrays (nHorzRes);      // allocate memory for the data
  InitializeLIVEInfo (&LIVEInfo);     // initialize the operation info
  InitializeGlobalDataArrays (0);     // initialize global data arrays
  InitializeTP112 ();                 // initialize the TP112 card
  InitializeMeasurementDependencies (&MeasDependencies); // setup the instrument

 /*
  * draw a darkgrey box around the place the signal will be displayed
  */
  setcolor(LIGHTGRAY);
  rectangle( OFFSIGFRAMEFROMLEFT+1,                 // X upper left corner
             OFFSIGFRAMEFROMTOP+1,                  // Y upper left corner
             OFFSIGFRAMEFROMLEFT + nNumFrameXPix+1, // X lower right corner
             OFFSIGFRAMEFROMTOP + nNumFrameYPix+4   // Y lower right corner
           );


 /*
  * perform continuous measurements in a loop, ended by a keypress
  */
  do
  {
    StartMeasuringTP112 (&MeasDependencies, &LIVEInfo);
  }
  while ( !kbhit() );

 /*
  * clean up
  */
  ResetTP112();  // Reset the TP112 card and the original timer interrupt
  closegraph();  // Restore the text mode screen
  FreeDataArrays();
  return(0);     // end the program without error code
}

/* NAME:
 *   void InitializeLIVEInfo (EQUIP_OVERALL_INFO* pLIVEInfo)
 * DESCRIPTION:
 *   This routine initializes the operating info for the scope
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pLIVEInfo   a pointer to the structure containing the info
 */
void InitializeLIVEInfo (EQUIP_OVERALL_INFO* pLIVEInfo)
{
  pLIVEInfo->MeasurementMode = ALT;    // CHAN1, CHAN2, ALT, CHOP, ADD,
                                       // COMPARE, XYPLOT, LAST_MEASMODE
  pLIVEInfo->OperationMode = OPLIVE;   // FREEZE, OPLIVE, LAST_OPERATE
} //InitializeLIVEInfo



/* NAME:
 *   void InitializeGlobalDataArrays( int nBeginRAMIndex )
 * DESCRIPTION:
 *   This routine initializes the global data arrays
 * INPUT PARAMETERS:
 *   nBeginRAMIndex   indicates the beginning of the RAM area
 * OUTPUT PARAMETERS:
 *   -
 */
void InitializeGlobalDataArrays (int nBeginRAMIndex)
{
  unsigned uRAMIndex;

  for ( uRAMIndex = nBeginRAMIndex;
	uRAMIndex <= (unsigned)(nNumSamples);
	uRAMIndex++
      )
  {
    *(pGlobalRAMDataCH1 + uRAMIndex) =
    *(pGlobalRAMDataCH2 + uRAMIndex) = (unsigned)MAXSAMPLRESHALF;
  }
} //InitializeGlobalDataArrays


/* NAME:
 *   void SetScopeBoxDimensions (int* pHorzRes)
 * DESCRIPTION:
 *   This routine determines the screen resolution and then sets the
 *   oscilloscope box dimensions.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pHorzRes   the horizontal screen resolution
 */
void SetScopeBoxDimensions (int* pHorzRes)
{
  int nVertRes;

  *pHorzRes        = getmaxx();
  nVertRes         = getmaxy();
  nScopeBoxYBottom = nVertRes - YROOMBUTTONS;
  nNumFrameYPix    = nScopeBoxYBottom - (OFFSIGFRAMEFROMTOP + OFFSIGFRAMEFROMBOT) + 1;
  nNumYPixPerDiv   = Round ((double)nNumFrameYPix / (double)(NUMYGRIDLINES));
  nScopeBoxXRight  = *pHorzRes - 5;
 /*
  * Determine GridXSpacing ---> rounded value
  */
  nNumFrameXPix = nScopeBoxXRight - 2 * OFFSIGFRAMEFROMLEFT + 1;
  nNumXPixPerDiv = Round ((double)nNumFrameXPix / (double)(NUMXGRIDLINES));

} //SetScopeBoxDimensions



/* NAME:
 *   void AllocateDataArrays (int nHorzRes)
 * DESCRIPTION:
 *   This routine allocates far memory for several data arrays
 *   This routine requires far data pointers, so the program has to be
 *   compiled in a Compact, Large or Huge memory model.
 * INPUT PARAMETERS:
 *   nHorzRes   the horizontal screen resolution
 * OUTPUT PARAMETERS:
 *   -
 */
void AllocateDataArrays (int nHorzRes)
{
 /*
  * Global Data Arrays
  */
  pGlobalRAMDataCH1 = (unsigned far*)farmalloc (2*nNumSamples + 100);
  pGlobalRAMDataCH2 = (unsigned far*)farmalloc (2*nNumSamples + 100);
  pGlobalRAMDataDig = (unsigned far*)farmalloc (2*nNumSamples + 100);
 /*
  * Drawing Array
  */
  pLocalRAMDataCH1 = (unsigned far*)farmalloc(nHorzRes*2);
  pLocalRAMDataCH2 = (unsigned far*)farmalloc(nHorzRes*2);
 /*
  * Wipe Array
  */
  pOldLocalRAMDataCH1 = (unsigned far*)farmalloc(nHorzRes*2);
  pOldLocalRAMDataCH2 = (unsigned far*)farmalloc(nHorzRes*2);
} //AllocateDataArrays


/* NAME:
 *   void FreeDataArrays
 * DESCRIPTION:
 *   This routine frees far memory for several data arrays
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void FreeDataArrays ( void )
{
 /*
  * Global Data Arrays
  */
  farfree( pGlobalRAMDataCH1 );
  farfree( pGlobalRAMDataCH2 );
  farfree( pGlobalRAMDataDig );
 /*
  * Drawing Array
  */
  farfree( pLocalRAMDataCH1 );
  farfree( pLocalRAMDataCH2 );

 /*
  * Wipe Array
  */
  farfree( pOldLocalRAMDataCH1 );
  farfree( pOldLocalRAMDataCH2 );

} //FreeDataArrays
