// File DRWSIG.C

/* System include files */
#include <graphics.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <stdlib.h>
#include <dos.h>

/* Custom include files */
#include "measure.h"
#include "ad_tpex.h"
#include "types.h"
#include "const.h"
#include "globvars.h"


#ifdef __cplusplus
unsigned char max (unsigned char value1, unsigned char value2);
unsigned char min (unsigned char value1, unsigned char value2);


unsigned char max (unsigned char value1, unsigned char value2)
{
 return ((value1 > value2) ? value1 : value2);
}

unsigned char min (unsigned char value1, unsigned char value2)
{
 return ((value1 < value2) ? value1 : value2);
}
#endif


/* Custom include files */
#include "addmath.h"
#include "globvars.h"
#include "const.h"



/* Local function definitions */
void RetrieveDataFromBoard (TP208_MEASDEP* pMeasSettings);
void ExtractSamples (EQUIP_OVERALL_INFO* pLIVEInfo);
void CompressData (unsigned far* pSignalDataBlock, unsigned far* pCompressData, float fGainValue, float fScrollBarOffset);
void OrdinaryCompress (ZOOMSAMPRANGE ZoomSampleRange, unsigned far* pSignalDataBlock, unsigned far* pCompressData, float fGainValue, float fScrollBarOffset, float fScreenAmplDivFactor);
void DisplaySignalData (int nChannel);
void DisplayOldSignalData (int nChannel);
void DrawNewSignal(unsigned far* pNewSignalData, unsigned far* pOldSignalData);
void DrawOldSignal (unsigned far* pOldSignalData);
void DisplayMeasuredSignals (EQUIP_OVERALL_INFO* pLIVEInfo);
void DisplayOldMeasuredSignals (EQUIP_OVERALL_INFO* pLIVEInfo);
void DrawLine (int nXBegin, int nYBegin, int nXEnd, int nYEnd);



void RetrieveDataFromBoard (TP208_MEASDEP* pMeasSettings)
{
 ReadSRAMData (pMeasSettings);
 bDataRetrieved = TRUE; // Now Samples can be extracted and signal initialized!!!
}//RetrieveDataFromBoard



/* Function assumes 32K RAM memory has been allocated */
void ExtractSamples (EQUIP_OVERALL_INFO* pLIVEInfo)
{
  switch (pLIVEInfo->MeasurementMode)
  {
    case COMPARE:
    case CHAN1:
	/*
	 * Now extract Signal samples, so that they fit in
	 * currently displayed Screen-dimensions;
	 *
	 * adjust amplitude values
	 */
	 CompressData (pGlobalRAMDataCH1, pLocalRAMDataCH1, 1.0, 0);
	 break;
    case CHAN2:
	/*
	 * Now extract Signal samples, so that they fit in
	 * currently displayed Screen-dimensions;
	 *
	 * adjust amplitude values
	 */
	 CompressData (pGlobalRAMDataCH2, pLocalRAMDataCH2, 1.0, 0);
	 break;
    case ALT:
    case ADD:
    case CHOP:
    case XYPLOT:
	 CompressData (pGlobalRAMDataCH1, pLocalRAMDataCH1, 1.0, 0);
	 CompressData (pGlobalRAMDataCH2, pLocalRAMDataCH2, 1.0, 0);
	 break;
  }
} //ExtractSamples



void CompressData ( unsigned far* pSignalDataBlock,
		    unsigned far* pCompressData,
		    float         fGainValue,
		    float 	  fScrollBarOffset
		  ) // Src-Dest
{
  ZOOMSAMPRANGE ZoomSampleRange;
  float fScreenAmplDivFactor = float(MAXSAMPLRES / nNumFrameYPix-1);

  ZoomSampleRange.lBeginSampVal = 0;
  ZoomSampleRange.lEndSampVal = nNumSamples;

 /*
  * Normal compression...
  */
  OrdinaryCompress ( ZoomSampleRange,
		     pSignalDataBlock,
		     pCompressData,
		     fGainValue,
		     fScrollBarOffset,
		     fScreenAmplDivFactor
		   );
} //CompressData



/* NAME:
 *   void OrdinaryCompress ( ZOOMSAMPRANGE ZoomSampleRange,
 *                           unsigned far* pSignalDataBlock,
 *                           unsigned far* pCompressData,
 *                           float         fGainValue,
 *                           float         fScrollBarOffset,
 *                           float         fScreenAmplDivFactor
 *                         )
 * DESCRIPTION:
 *   This routine 'compresses' (= scales) the sampled data of a channel to
 *   values within the available range for the current display.
 * INPUT PARAMETERS:
 *   ZoomSampleRange       Indicates which part has to be displayed
 *   pSignalDataBlock      The sampled, uncompressed data
 *   fGainValue
 *   fScrollBarOffset
 *   fScreenAmplDivFactor
 * OUTPUT PARAMETERS:
 *   pCompressData         The compressed data
 *   -
 */
void OrdinaryCompress ( ZOOMSAMPRANGE ZoomSampleRange,
			unsigned far* pSignalDataBlock,
			unsigned far* pCompressData,
			float         fGainValue,
			float         fScrollBarOffset,
			float         fScreenAmplDivFactor
		      )
{
  int   nCompressDataIndex;
  float fGainDepScrollBarValue = fScrollBarOffset*fGainValue;
  float fAdjNullOffset         = MAXSAMPLRESHALF/fScreenAmplDivFactor;

  float fSkipGlobalDataOffset, fSkipGlobalDataCount =
	    float(ZoomSampleRange.lBeginSampVal);

  fSkipGlobalDataOffset =
   (  (float)(ZoomSampleRange.lEndSampVal - ZoomSampleRange.lBeginSampVal) /
      nNumFrameXPix
   );

  for ( nCompressDataIndex = 0;
	nCompressDataIndex < nNumFrameXPix;
	nCompressDataIndex++
      )
  {
    *(pCompressData + nCompressDataIndex) =
       -Round(
	       (double)(
			 (  *( pSignalDataBlock +
			       Round( double( fSkipGlobalDataCount ) )
			     ) /
			   fScreenAmplDivFactor
			 ) -
			 fAdjNullOffset
		       ) *
	       fGainValue + fGainDepScrollBarValue + fAdjNullOffset
	     );
    fSkipGlobalDataCount += fSkipGlobalDataOffset;
  }
} //OrdinaryCompress


/* NAME:
 *   void DisplaySignalData (int nChannel)
 * DESCRIPTION:
 *   This routine displays the signal data of one channel. The data is
 *   currently in the 32K RAM buffer
 * INPUT PARAMETERS:
 *   nChannel    indicating which channel to display
 * OUTPUT PARAMETERS:
 *   -
 */
void DisplaySignalData (int nChannel)
{
  unsigned far* pCompressData;
  unsigned far* pOldSignalData;

  switch (nChannel)
  {
    case CHANNEL1:
	 pOldSignalData = pOldLocalRAMDataCH1;
	 pCompressData  = pLocalRAMDataCH1;
	 setcolor (EGA_YELLOW);
	 break;
    case CHANNEL2:
	 pOldSignalData = pOldLocalRAMDataCH2;
	 pCompressData  = pLocalRAMDataCH2;
	 setcolor (EGA_LIGHTGREEN);
	 break;
  }
  if (bFirstLIVE)
  {
    CompressData (pGlobalRAMDataCH1, pOldSignalData, 1.0, 0.0);
    DrawOldSignal (pOldSignalData);
  }
  DrawNewSignal (pCompressData, pOldSignalData);

} //DisplaySignalData



/* NAME:
 *   void DisplayOldSignalData (int nChannel)
 * DESCRIPTION:
 *   This routine displays the old signal data of one channel. The data is
 *   currently in the 32K RAM buffer
 * INPUT PARAMETERS:
 *   nChannel    indicating which channel to display
 * OUTPUT PARAMETERS:
 *   -
 */
void DisplayOldSignalData (int nChannel)
{
  unsigned far* pOldSignalData;

  switch (nChannel)
  {
    case CHANNEL1:
	 pOldSignalData = pOldLocalRAMDataCH1;
	 setcolor (EGA_YELLOW);
	 break;
    case CHANNEL2:
	 pOldSignalData = pOldLocalRAMDataCH2;
	 setcolor (EGA_LIGHTGREEN);
	 break;
  }
  DrawOldSignal (pOldSignalData);

} //DisplayOldSignalData


/* NAME:
 *   void DrawNewSignal (unsigned far* pNewSignalData, unsigned far* pNewSignalData)
 * DESCRIPTION:
 *   This routine draws the new signal, after first erasing the old signal
 * INPUT PARAMETERS:
 *   pNewSignalData   a pointer to the data to display
 *   pOldSignalData   a pointer to the data to erase
 * OUTPUT PARAMETERS:
 *   -
 */
void DrawNewSignal(unsigned far* pNewSignalData, unsigned far* pOldSignalData)
{
  int nSampIndex;
  unsigned nPointYNewBegin, nPointYNewEnd;
  unsigned nPointYOldBegin, nPointYOldEnd;

 /*
  * Retrieve first Y-value Old signal
  */
  nPointYOldBegin = *pOldSignalData;
  nPointYNewBegin = *pNewSignalData;
  *pOldSignalData = nPointYNewBegin;

  for ( nSampIndex = 1; nSampIndex < nNumFrameXPix; nSampIndex++ )
  {
    /*
     * First erase previous signal!!!
     */
     nPointYOldEnd = *(pOldSignalData + nSampIndex);
     DrawLine( nSampIndex-1, -nPointYOldBegin, nSampIndex-1, -nPointYOldEnd );
     nPointYOldBegin = nPointYOldEnd;

     nPointYNewEnd = *(pNewSignalData + nSampIndex);
    /*
     * pOldSignal = pNewSignal ---> for next new signal
     */
     *(pOldSignalData + nSampIndex) = nPointYNewEnd;
     DrawLine( nSampIndex-1, -nPointYNewBegin, nSampIndex-1, -nPointYNewEnd );
     nPointYNewBegin = nPointYNewEnd;
  }
} //DrawNewSignal


/* NAME:
 *   void DrawOldSignal (unsigned far* pOldSignalData)
 * DESCRIPTION:
 *   This routine draws the old signal
 * INPUT PARAMETERS:
 *   pOldSignalData   a pointer to the data to display
 * OUTPUT PARAMETERS:
 *   -
 */
void DrawOldSignal (unsigned far* pOldSignalData)
{
  unsigned nPointYOldBegin, nPointYOldEnd;
  int nSampIndex;

  nPointYOldBegin = *pOldSignalData;
  for ( nSampIndex = 1; nSampIndex < nNumFrameXPix; nSampIndex++ )
  {
     nPointYOldEnd = *(pOldSignalData + nSampIndex);
     DrawLine( nSampIndex-1, -nPointYOldBegin, nSampIndex-1, -nPointYOldEnd );
     nPointYOldBegin = nPointYOldEnd;
  }
} //DrawOldSignal



/* NAME:
 *   void DisplayMeasuredSignals (EQUIP_OVERALL_INFO* pLIVEInfo)
 * DESCRIPTION:
 *   This routine displays the measured signals
 * INPUT PARAMETERS:
 *   pLIVEInfo   a structure, containing info about what to display
 * OUTPUT PARAMETERS:
 *   -
 */
void DisplayMeasuredSignals (EQUIP_OVERALL_INFO* pLIVEInfo)
{
  #define CLIP_ON  1  /* activates clipping */
  #define CLIP_OFF 0  /* activates clipping */

  setviewport ( OFFSIGFRAMEFROMLEFT+2,                // X upper left corner
		OFFSIGFRAMEFROMTOP+2,                 // Y upper left corner
		OFFSIGFRAMEFROMLEFT + nNumFrameXPix,  // X lower right corner
		OFFSIGFRAMEFROMTOP + nNumFrameYPix-1, // Y lower right corner
		CLIP_OFF                              // clipping
	      );
 /*
  * select XOR drawing mode
  */
  setwritemode(XOR_PUT);

  switch (pLIVEInfo->MeasurementMode)
  {
    case CHAN1: // Default Mode
      /*
       * Wipe and draw...
       */
       DisplaySignalData( CHANNEL1 );
       break;
    case CHAN2:
       DisplaySignalData( CHANNEL2 );
       break;
    case ALT:
    case CHOP:
       DisplaySignalData( CHANNEL1 );
       DisplaySignalData( CHANNEL2 );
       break;
   /*
    * Must be extended!!!
    */
  }
  bFirstLIVE = FALSE;

} //DisplayMeasuredSignals



/* NAME:
 *   void DisplayOldMeasuredSignals (EQUIP_OVERALL_INFO* pLIVEInfo)
 * DESCRIPTION:
 *   This routine displays the old measured signals
 * INPUT PARAMETERS:
 *   pLIVEInfo   a structure, containing info about what to display
 * OUTPUT PARAMETERS:
 *   -
 */
void DisplayOldMeasuredSignals (EQUIP_OVERALL_INFO* pLIVEInfo)
{
  #define CLIP_ON  1   /* activates clipping in */
  #define CLIP_OFF 0   /* activates clipping in */

 /*
  * select XOR drawing mode
  */
  setwritemode(XOR_PUT);
  setviewport (
		OFFSIGFRAMEFROMLEFT+2,               // X upper left corner
		OFFSIGFRAMEFROMTOP+2,                // Y upper left corner
		OFFSIGFRAMEFROMLEFT + nNumFrameXPix, // X lower right corner
		OFFSIGFRAMEFROMTOP + nNumFrameYPix-1,// Y lower right corner
		CLIP_OFF                             // clipping
	      );

  switch (pLIVEInfo->MeasurementMode)
  {
    case CHAN1: // Default Mode
	/*
	 * Only Draw...
	 */
	 DisplayOldSignalData (CHANNEL1);
	 break;
    case CHAN2:
	 DisplayOldSignalData (CHANNEL2);
	 break;
    case ALT:
	 DisplayOldSignalData (CHANNEL1);
	 DisplayOldSignalData (CHANNEL2);
	 break;
   /*
    * Must be extended!!!
    */
  }
} //DisplayOldMeasuredSignal


/* NAME:
 *   void DrawLine (int nXBegin, int nYBegin, int nXEnd, int nYEnd)
 * DESCRIPTION:
 *   This routine draws a line from (nXBegin, nYBegin) to ( nXEnd, nYEnd )
 * INPUT PARAMETERS:
 *   nXBegin     the X coordinate of the start point
 *   nYBegin     the Y coordinate of the start point
 *   nXEnd       the X coordinate of the end point
 *   nYEnd       the Y coordinate of the end point
 * OUTPUT PARAMETERS:
 *   -
 */
void DrawLine (int nXBegin, int nYBegin, int nXEnd, int nYEnd)
{
  line( nXBegin, nYBegin, nXEnd, nYEnd );
} //DrawLine
