/* File Ad_Tp508.c */

/* System include files */

extern "C" {
 #include <dos.h>
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
 #include <string.h>
 #include <stdio.h>
}

/* Custom include files */
#include "ad_tp508.h"
#include "addmath.h"
#include "globvars.h"
#include "types.h"
#include "const.h"

#define PGA_BLNK        64
#define REF_SCLT        32
#define PGA_SLCT        16


/* Global Variable Declarations */
// Following buffers specify where to find the CONTROL and DATA registers on the I/O map */
BYTE ContReg[] = {0, 4, 6};
BYTE DataReg[] = {1, 5, 7};

BYTE Level1_Con_Ad[]     = {0, 0};
BYTE Level2_Con_Ad[]     = {1, 1};
BYTE Trig_Niv_Con_Ad[]   = {2, 2};
BYTE Pga_Con_Ad[]        = {3, 3};
BYTE Data_Nul_Con_Ad[]   = {4, 4};
BYTE Irq_Con_Ad[]        = {5, 5};
BYTE Trig_Reset_Con_Ad[] = {6, 6};

/* Variables for the DATA register ---> are being initialized */
BYTE Post_Lo_Reg        = 0;
BYTE Post_Hi_Reg        = 0;
BYTE Output_Reg         = 0;
BYTE TimeBase_Reg       = 0;
BYTE Mode_Reg           = 0;
BYTE Reset_Reg          = 0;
BYTE Count_Reg          = 0;
BYTE Write_TimeBase_Reg = 0;

/* Register Contants Variables */
BYTE Level1_Reg_Ad[]     = {0, 0};
BYTE Level2_Reg_Ad[]     = {0, 0};
BYTE Trig_Niv_Reg_Ad[]   = {0, 0};
BYTE Pga_Reg_Ad[]        = {0, 0};
BYTE Trig_Reset_Reg_Ad[] = {0, 0};

int Ad_PortAddress = 0x288; /* 0x300; */

BYTE Ref_Value[NUMCHANNELS][15] =
            {
              {128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128},
              {128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128}
            };
float DA_Value[NUMCHANNELS][15] =
            {
              {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
              {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
            };
float Offset_AD[NUMCHANNELS] = {-1, -1};
long int Ch_Switch_Value[NUMCHANNELS][15] =
{
  {
    PGA1_GAIN_1_10 + PGA1_GAIN_4 + PGA1_ATT_1,
    PGA1_GAIN_1_10 + PGA1_GAIN_2 + PGA1_ATT_1,
    PGA1_GAIN_1_10 + PGA1_GAIN_1 + PGA1_ATT_1,
    PGA1_GAIN_4 + PGA1_ATT_1,
    PGA1_GAIN_2 + PGA1_ATT_1,
    PGA1_GAIN_1 + PGA1_ATT_1,
    PGA1_GAIN_4 + PGA1_ATT_10,
    PGA1_GAIN_2 + PGA1_ATT_10,
    PGA1_GAIN_1 + PGA1_ATT_10,
    PGA1_GAIN_4 + PGA1_ATT_100,
    PGA1_GAIN_2 + PGA1_ATT_100,
    PGA1_GAIN_1 + PGA1_ATT_100
  },

  {
    PGA2_GAIN_1_10 + PGA2_GAIN_4 + PGA2_ATT_1 + PGA2_ON,
    PGA2_GAIN_1_10 + PGA2_GAIN_2 + PGA2_ATT_1 + PGA2_ON,
    PGA2_GAIN_1_10 + PGA2_GAIN_1 + PGA2_ATT_1 + PGA2_ON,
    PGA2_GAIN_4 + PGA2_ATT_1 + PGA2_ON,
    PGA2_GAIN_2 + PGA2_ATT_1 + PGA2_ON,
    PGA2_GAIN_1 + PGA2_ATT_1 + PGA2_ON,
    PGA2_GAIN_4 + PGA2_ATT_10 + PGA2_ON,
    PGA2_GAIN_2 + PGA2_ATT_10 + PGA2_ON,
    PGA2_GAIN_1 + PGA2_ATT_10 + PGA2_ON,
    PGA2_GAIN_4 + PGA2_ATT_100 + PGA2_ON,
    PGA2_GAIN_2 + PGA2_ATT_100 + PGA2_ON,
    PGA2_GAIN_1 + PGA2_ATT_100 + PGA2_ON
  }
};


/* Local function declarations */
void SetReferenceVoltageAD(int Value1, int Value2);
void FillReferenceAD(int nValue);
void SetPGA(BOOL bAC_DCCH1, BOOL bAC_DCCH2, long lValue);
void AdjustOffsetDA(int nChannelIndex, float fValue);
void FillLevelRegAd(int nHelp, BYTE byLevel, int nCh);
BOOL ReadEOCBit(void);
void SetPostCount(WORD wPostCount);
void SetTrigLevel(SLOPE Slope, BYTE byLevel, BYTE byHysteresis);
void SetTrigBit(void);
void SetTimeBase(float* fFreq);
void SetCalOutput(float* fFreq);
void ClearSRAM(void);
void GoToSRAMPos(long lSRAMPos);
void ReadSRAMData(TP508_MEASDEP* pMeasSettings);
void MeasureTP508(TP508_MEASDEP* pMeasSettings);
void DetermineOffset(void);
void FillPGAStands(long int PGAStand [NUMCHANNELS+1][6]);
long int DetermineMean(int nChannelIndex, int nStand, int nDeviation, int nBeginValue);
unsigned long CheckMeanMeasurementData(unsigned long lMeasValue, int nStand, int nDeviation, int nNumGain, int nChannelIndex, int nBeginValue);
void DetermineGain (void);
void DetermineGainCHX(int nChannelIndex, long int PGAStand [NUMCHANNELS + 1][6]);
void Calibrate(BOOL* pbCalError);
void CalibrateOnce(BOOL* pbCalError);
BOOL Calibrate_TP508(void);
void DetermineTime(void);


/*
 * Ad_Tp508.c Local function definitions...
 */

/* NAME:
 *   void SetReferenceVoltageAD (int nValue1, int nValue2)
 * DESCRIPTION:
 *   This routine sets the reference voltages for the ADC's
 * INPUT PARAMETERS:
 *   nValue1   the value for the first reference voltage
 *   nValue2   the value for the second reference voltage
 * OUTPUT PARAMETERS:
 *   -
 */
void SetReferenceVoltageAD(int nValue1, int nValue2)
{
 BYTE byS_Clck, byS_Data;

/*
 * select the serial register of CONTROL REGISTER 2 to [Ad-PortAddress+5]
 */
 outportb( Ad_PortAddress + 4, (unsigned char)3 );
 byS_Data = 0;
 byS_Clck = 0;
 outportb( Ad_PortAddress+5, (unsigned char)(byS_Data+byS_Clck+PGA_BLNK) );
 outportb( Ad_PortAddress+5, (unsigned char)(byS_Data+byS_Clck+REF_SCLT+PGA_BLNK) );

 byS_Clck = 2;
 outportb( Ad_PortAddress+5, (unsigned char)(byS_Data+byS_Clck+REF_SCLT+PGA_BLNK) );

/*
 * For Reference Value 1
 */
 FillReferenceAD( nValue1 );

/*
 * For Reference Value 1
 */
 FillReferenceAD( nValue2 );

 byS_Data = 0;
 byS_Clck = 0;
 outportb( Ad_PortAddress+5, (unsigned char)(byS_Data+byS_Clck+REF_SCLT+PGA_BLNK) );
 outportb( Ad_PortAddress+5, PGA_BLNK );

} //SetReferenceVoltageAD



/* NAME:
 *   void FillReferenceAD (int nValue)
 * DESCRIPTION:
 *   This routine writes one AD reference value.
 *   Serial data and clock are generated
 * INPUT PARAMETERS:
 *    nValue    the value to write
 * OUTPUT PARAMETERS:
 *    -
 */
void FillReferenceAD (int nValue)
{

 int nCount;
 WORD wHelp = (int) (exp( (float)7 * (float)log(2) )+ 0.5); //128;
 BYTE byS_Clck, byS_Data;

 for (nCount = 0; nCount <= 7; nCount++)
 {
  if ((nValue & wHelp) == 0)
   byS_Data = 0;
  else
   byS_Data = 1;

  byS_Clck = 0;
  outportb( Ad_PortAddress+5, (unsigned char)(byS_Data+byS_Clck+REF_SCLT+PGA_BLNK) );
  byS_Clck = 2;
  outportb( Ad_PortAddress+5, (unsigned char)(byS_Data+byS_Clck+REF_SCLT+PGA_BLNK) );
  wHelp = div(wHelp, 2).quot;
 }

} //FillReferenceAD



/* NAME:
 *   void SetPGA (BOOL bAC_DCCH1, BOOL bAC_DCCH2, long lValue)
 * DESCRIPTION:
 *   This routine converts the switch data and writes it to the hardware
 * INPUT PARAMETERS:
 *   bAC_DCCH1   coupling setting ch1
 *   bAC_DCCH2   coupling setting ch2
 *   lValue      the other switches (attenuators)
 * OUTPUT PARAMETERS:
 *   -
 */
void SetPGA (BOOL bAC_DCCH1, BOOL bAC_DCCH2, long lValue)
{
 const nSerialDataLength = 19;   // 20 bytes to send
 BYTE S_Clck, S_Data;
 long int lHelp;
 int nCount;

 if (!bAC_DCCH1)
  lValue += PGA1_AC_DC;
 if (!bAC_DCCH2)
  lValue += PGA2_AC_DC;
/*
 * select the serial register of CONTROL REGISTER 2 to [Ad-PortAddress+5]
 */
 outportb( Ad_PortAddress+4, (unsigned char)3 );
 lHelp =  1048576;  // this is 0x100000
 S_Clck = 0;
 S_Data = 0;
 outportb( Ad_PortAddress+5, (unsigned char)S_Data+S_Clck+PGA_BLNK );

 for (nCount = 0; nCount <= nSerialDataLength; nCount++)
 {
  if ((lValue & lHelp) == 0)
    S_Data = 0;
  else
    S_Data = 1;
  S_Clck = 0;
  outportb( Ad_PortAddress+5, (unsigned char)S_Data+S_Clck+PGA_BLNK );
  S_Clck = 2;
  outportb( Ad_PortAddress+5, (unsigned char)S_Data+S_Clck+PGA_BLNK );
  lHelp = ldiv( lHelp, 2 ).quot;
 }
 outportb( Ad_PortAddress+5, (unsigned char)PGA_SLCT+PGA_BLNK );
 outportb( Ad_PortAddress+5, (unsigned char)PGA_BLNK );

/*
 * Wait X msec for switching PGA
 */
 delay (PGA_SWITCHINGTIME);

} //SetPGA



/* NAME:
 *   void AdjustOffsetDA (int nChannelIndex, float fValue)
 * DESCRIPTION:
 *   This routine adjusts the value for the offset DA outputs
 * INPUT PARAMETERS:
 *   nChannelIndex   indicates which Offset DA to adjust
 *   fValue          the value to write to the offset DA
 * OUTPUT PARAMETERS:
 *   -
 */
void AdjustOffsetDA (int nChannelIndex, float fValue)
{
 const BYTE Ad_Slct [NUMCHANNELS] = {4, 8};

 BYTE S_Clck, S_Data;
 WORD wHelp;
 int nCount, nData;

/*
 * select the serial register of CONTROL REGISTER 2 to [Ad-PortAddress+5]
 */
 outportb( Ad_PortAddress+4, (unsigned char)3 );

/*
 * scale the fValue to an int value which ranges from 0 to 4095
 */
 fValue = ((fValue/5) * 2048) + 2048;
 if (fValue > 4095)
  fValue = 4095;
 if (fValue < 0)
  fValue = 0;
 nData = Round ((double)fValue);

 nData = nData ^ 2048;

 wHelp = 32768L;
 S_Clck = 2;
 S_Data = 0;
 outportb( Ad_PortAddress+5,
           (unsigned char)(S_Data+S_Clck+Ad_Slct[nChannelIndex]+PGA_BLNK)
         );

 for (nCount = 1; nCount <= 16; nCount++)
 {
  if ((nData & wHelp) == 0)
    S_Data = 0;
  else
    S_Data = 1;
  S_Clck = 2;
  outportb( Ad_PortAddress+5,
            (unsigned char)(S_Data+S_Clck+Ad_Slct[nChannelIndex]+PGA_BLNK)
          );
  S_Clck = 0;
  outportb( Ad_PortAddress+5,
            (unsigned char)(S_Data+S_Clck+Ad_Slct[nChannelIndex]+PGA_BLNK)
          );
  wHelp = ldiv (wHelp, 2).quot;
 }
 outportb( Ad_PortAddress+5, (unsigned char)PGA_BLNK );
} //AdjustOffsetDA



/* NAME:
 *   BOOL ReadEOCBit( void )
 * DESCRIPTION:
 *   This routine reads the End Of Conversion bit
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   returnvalue is the End Of Conversion bit
 */
BOOL ReadEOCBit( void )
{

 return( BOOL(( inportb( int(Ad_PortAddress + 2) ) ) & 0x01 ));

} //ReadEOCBit



/* NAME:
 *   void SetPostCount (WORD wPostCount)
 * DESCRIPTION:
 *   This routine sets the value for the post counter
 * INPUT PARAMETERS:
 *   wPostCount  contains the new value for the post counter
 * OUTPUT PARAMETERS:
 *   -
 */
void SetPostCount (WORD wPostCount)
{
/*
 * split the post counter value in lower and higher byte
 */
 Post_Lo_Reg = LOBYTE( wPostCount );
 Post_Hi_Reg = HIBYTE( wPostCount );

/*
 * select the POST-LO register and store the lower byte of the value into it
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)POST_LO_CON );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)Post_Lo_Reg );
/*
 * select the POST-HI register and store the higher byte of the value into it
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)POST_HI_CON );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)Post_Hi_Reg );

}//SetPostCount



/* NAME:
 *   void SetTrigLevel( SLOPE Slope, BYTE byLevel, BYTE byHysteresis )
 * DESCRIPTION:
 *   This routine sets the trigger level
 * INPUT PARAMETERS:
 *   Slope        the triggerslope
 *   byLevel      the value for the trigger level
 *   byHysteresis the trigger hysteresis
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTrigLevel( SLOPE Slope, BYTE byLevel, BYTE byHysteresis )
{
  int nHelp, nCh;

  for (nCh = 0; nCh <= 1; nCh++)
  {
    if (Slope == POS) // Positive Slope
    {
      nHelp = byLevel - byHysteresis;
      FillLevelRegAd( nHelp, byLevel, nCh );
      Trig_Niv_Reg_Ad[nCh] = 5; // 0000 0101
    }
    else // Negative Slope
    {
      nHelp = byLevel + byHysteresis;
      FillLevelRegAd( nHelp, byLevel, nCh );
      Trig_Niv_Reg_Ad[nCh] = 6; // 0000 0110
    }
    outportb( Ad_PortAddress+ContReg[nCh+1], Trig_Niv_Con_Ad[nCh] );
    outportb( Ad_PortAddress+DataReg[nCh+1], Trig_Niv_Reg_Ad[nCh] );

    outportb( Ad_PortAddress+ContReg[nCh+1], Level1_Con_Ad[nCh] );
    outportb( Ad_PortAddress+DataReg[nCh+1], Level1_Reg_Ad[nCh] );
    outportb( Ad_PortAddress+ContReg[nCh+1], Level2_Con_Ad[nCh] );
    outportb( Ad_PortAddress+DataReg[nCh+1], Level2_Reg_Ad[nCh] );
  }

} //SetTrigLevel



/* NAME:
 *   void FillLevelRegAd (int nHelp, BYTE byLevel, int nCh)
 * DESCRIPTION:
 *   This routine fills the trigger level registers
 * INPUT PARAMETERS:
 *   nHelp    a help variable
 *   byLevel  contains the triggerlevel
 *   nCh      channel indicator
 * OUTPUT PARAMETERS:
 *   -
 */
void FillLevelRegAd (int nHelp, BYTE byLevel, int nCh)
{

 if (nHelp > 255)
   nHelp = 255;
 if (nHelp < 0)
   nHelp = 0;
 Level1_Reg_Ad[nCh] = nHelp;
 nHelp = byLevel;

 if (nHelp > 255)
   nHelp = 255;
 if (nHelp < 0)
   nHelp = 0;
 Level2_Reg_Ad[nCh] = nHelp;

}//FillLevelRegAd



/* NAME:
 *   void SetTrigBit( void )
 * DESCRIPTION:
 *   This routine sets the trigger bit.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTrigBit (void)
{
 outportb( Ad_PortAddress,   (unsigned char)7 ); // trigger select register
 outportb( Ad_PortAddress+1, (unsigned char)2 ); // triggering direct

//  outportb( Ad_PortAddress+4, (unsigned char)2 );  // trigger level register
//  outportb( Ad_PortAddress+5, (unsigned char)13 ); // AND trigger

}//SetTrigBit



/* NAME:
 *   void SetTimeBase (float* fFreq)
 * DESCRIPTION:
 *   This routine sets the timebase register. Some frequencies can be made
 *   directly by dividing the clock twice or four times. For other frequencies
 *   the 8254 has to be programmed with another value.
 * INPUT PARAMETERS:
 *   fFreq    the frequency to make
 * OUTPUT PARAMETERS:
 *   -
 */
void SetTimeBase (float* fFreq)
{
 WORD wDivFactor;
 BYTE HighBytewDiv, LowBytewDiv;
 float fInputFreq;

 if (*fFreq >= 25e6)
 {
  TimeBase_Reg = 0+32;  // 0010 0000
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);
  return; // Exit procedure
 }
 if (*fFreq >= 12.5e6)
 {
  TimeBase_Reg = 4+32;  // 0010 0100
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);
  return;
 }
 if (*fFreq >= 5e6)
 {
  TimeBase_Reg = 8+32;  // 0010 1000
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);
  return;
 }
 fInputFreq = 5e6;

/*
 * now a special division factor has to be calculated
 */
 if (*fFreq > 5e6)
 {
   *fFreq = 5e6;
 }
 if (*fFreq < 80)
 {
   fInputFreq = 5e4;
 }
 wDivFactor = (WORD)Round (double(fInputFreq/(*fFreq)));
 *fFreq = fInputFreq/wDivFactor;

/*
 * Initialize timer IC 8254, control word A1, A0=11
 */
 TimeBase_Reg = 3+32;

/*
 * Activate TimeBase Register
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)54);

/*
 *  00 [=select counter 0]
 *  11 [=write LSBYTE first]
 * 011 [=mode 3, divide by n counter]
 *   0 [=16-bits counter]
 */

/*
 * Load DivFactor
 */
 TimeBase_Reg = 0+32;

/*
 * Activate TimeBase Register
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

 outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);

 LowBytewDiv = LOBYTE(wDivFactor); // & 0x00FF;
 outportb( Ad_PortAddress + DataReg[0], (unsigned char) LowBytewDiv);
 HighBytewDiv = HIBYTE(wDivFactor); // & 0xFF00;
 outportb( Ad_PortAddress + DataReg[0], (unsigned char) HighBytewDiv);

 if (*fFreq < 80)
 {
 /*
  * Initialize timer IC 8254, control word A1,A0=11
  */
  TimeBase_Reg=3+32;

 /*
  * Activate TimeBase Register
  */
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

  outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)118);
 /*
  *  01 [=select counter 1]
  *  11 [=write LSBYTE first]
  * 011 [=mode 3, divide by n counter]
  *   0  [=16-bits counter]
  */

 /*
  * Load DivFactor
  */
  TimeBase_Reg = 1+32;

 /*
  * Activate TimeBase Register
  */
  outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

  outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)LOBYTE (100));
  outportb( Ad_PortAddress + DataReg[0], (unsigned char)HIBYTE (100));
 }

/*
 * Select Clock Input
 */
 if (*fFreq < 80)
  TimeBase_Reg = 16+32; // Select clck input b
 else
  TimeBase_Reg = 12+32; // Select clck input a
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

} //SetTimeBase


/* NAME:
 *   void SetCalOutput (float* fFreq)
 * DESCRIPTION:
 *   this routine sets the frequency for the calibration output
 *   and switch it on
 * INPUT PARAMETERS:
 *   fFreq  the frequency for the output
 * OUTPUT PARAMETERS:
 *   -
 */
void SetCalOutput (float* fFreq)
{
 WORD wDivFactor;
 float fInputFreq;

 fInputFreq = 5e6;

/*
 * Determine DivFactor
 */
 if (*fFreq > 1e5)
   *fFreq = 1e5;
 wDivFactor = (WORD)Round ((double)(fInputFreq/(*fFreq)));
 *fFreq = fInputFreq/wDivFactor;

/*
 * InitializeTimer IC8254, control word A1, A0=11
 */
 TimeBase_Reg=3+32;

/*
 * Activate TimeBase Register
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

 outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)182);
/*
 *   10 [=select counter 2]
 *   11 [=write LSBYTE first]
 *  011 [=mode 3, divide by n counter]
 *    0 [=16-bits counter]
 */

/*
 * Load DivFactor in counter 2
 */
 TimeBase_Reg = 2+32;

/*
 * Activate TimeBase Register
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

 outportb( Ad_PortAddress + ContReg[0], (unsigned char)WRITE_TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)LOBYTE (wDivFactor));
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)HIBYTE (wDivFactor));

 TimeBase_Reg=3+32;

/*
 * Activate TimeBase Register
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)TIMEBASE_CON);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)TimeBase_Reg);

} //SetCalOutput



/* NAME:
 *   void ClearSRAM (void)
 * DESCRIPTION:
 *   This routine clears the SRAM
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void ClearSRAM (void)
{
 int nCount;
 float fFreq;

/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)3);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);

 SetPostCount (32768L + 1);

 for (nCount = 0; nCount <= 1; nCount++)
 {
  outportb( Ad_PortAddress + ContReg[nCount + 1], (unsigned char)4);
  outportb( Ad_PortAddress + DataReg[nCount + 1], (unsigned char)1);
 }

/*
 * Set fastest TimeBase
 */
 fFreq = 25e6;
 SetTimeBase (&fFreq);

/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)3);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)14);//1110

/*
 * Reset Post Counter and Reset Address Counter
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)4);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);

/*
 * Turn CLCK On
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)3);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)15);//1111

} // ClearSRAM



/* NAME:
 *   void GoToSRAMPos (long lSRAMPos)
 * DESCRIPTION:
 *
 * INPUT PARAMETERS:
 *   lSRAMPos  to position in the SRAM to go to
 * OUTPUT PARAMETERS:
 *   -
 */
void GoToSRAMPos (long lSRAMPos)
{
 float fFreq;

/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)3);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);

 SetPostCount (lSRAMPos);
 fFreq = 25e6;
 SetTimeBase (&fFreq);

/*
 * Reset post
 */
 outportb( Ad_PortAddress+0, (unsigned char)4);
 outportb( Ad_PortAddress+1, (unsigned char)0);

/*
 * Set Trig at no-Trig
 */
 outportb( Ad_PortAddress+0, (unsigned char)7);
 outportb( Ad_PortAddress+1, (unsigned char)2);

/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress+0, (unsigned char)3);
 outportb( Ad_PortAddress+1, (unsigned char)2);//0010

/*
 * Turn on Clck
 */
 outportb( Ad_PortAddress+0, (unsigned char)3);
 outportb( Ad_PortAddress+1, (unsigned char)3);//0011

/*
 * Wait until TrigBit == '1'
 */
 do
 {
   // just do nothing, wait
 }
 while ((inportb (Ad_PortAddress+2) & 0x01) == 0);

 outportb( Ad_PortAddress + 0, (unsigned char)3);
 outportb( Ad_PortAddress + 1, (unsigned char)0);

}  //GoToSRAMPos



/* NAME:
 *   void ReadSRAMData (TP508_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine reads the SRAM data to the data buffer in the PC RAM
 * INPUT PARAMETERS:
 *   pMeasSettings  contains measurement instrument settings
 * OUTPUT PARAMETERS:
 *   -
 */
void ReadSRAMData (TP508_MEASDEP* pMeasSettings)
{
 int nSampleIndex;
 long int lHelp, lCounter_Lo, lCounter;
 unsigned int far* pRAMDataCH1;
 unsigned int far* pRAMDataCH2;

/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress + ContReg[0], (unsigned char)3);
 outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);

/*
 * Read Address Counter
 */
 lCounter_Lo = (unsigned char)inportb (Ad_PortAddress + 0);
 lCounter = (long)inportb (Ad_PortAddress + 1)*256 + lCounter_Lo;
 lCounter = (lCounter & RAMSIZE_TP508);

 lHelp = lCounter - pMeasSettings->NumPostSamples - pMeasSettings->NumPreSamples;

 if (lHelp < 0)
   lHelp+=RAMSIZE_TP508;
 GoToSRAMPos (lHelp);

 switch (pMeasSettings->MeasurementMode)
 {
  case CHAN1:
    for ( nSampleIndex = 0;
          nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
          nSampleIndex++
        )
    {
      *(pGlobalRAMDataCH1 + nSampleIndex) = (inportb(Ad_PortAddress+4)) * MULPLYFACTOR_TP508;
     /*
      * Increment Address Counter
      */
      outportb( Ad_PortAddress + ContReg[0], (unsigned char)5);
      outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);
    }
    break;
  case CHAN2:
    for ( nSampleIndex = 0;
          nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
          nSampleIndex++
        )
    {
      *(pGlobalRAMDataCH2 + nSampleIndex) = (inportb (Ad_PortAddress+6)) * MULPLYFACTOR_TP508;
     /*
      * Increment Address Counter
      */
      outportb( Ad_PortAddress + ContReg[0], (unsigned char)5);
      outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);
    }
    break;
  case ALT:
  case ADD:
  case CHOP:
  case XYPLOT:
    for ( nSampleIndex = 0;
          nSampleIndex < (pMeasSettings->NumPreSamples + pMeasSettings->NumPostSamples);
          nSampleIndex++
         )
    {
      *(pGlobalRAMDataCH1 + nSampleIndex) = (inportb (Ad_PortAddress + 4)) * MULPLYFACTOR_TP508;
      *(pGlobalRAMDataCH2 + nSampleIndex) = (inportb (Ad_PortAddress + 6)) * MULPLYFACTOR_TP508;
     /*
      * Increment Address Counter
      */
      outportb( Ad_PortAddress + ContReg[0], (unsigned char)5);
      outportb( Ad_PortAddress + DataReg[0], (unsigned char)0);
    }
    break;
 }

/*
 * Reset TrigChannel 1
 */
 outportb( Ad_PortAddress + ContReg[1], (unsigned char)5);
 outportb( Ad_PortAddress + DataReg[1], (unsigned char)0);
/*
 * Reset TrigChannel 2
 */
 outportb( Ad_PortAddress + ContReg[2], (unsigned char)5);
 outportb( Ad_PortAddress + DataReg[2], (unsigned char)0);
 ClearSRAM (); // For next measurement

} //ReadSRAMData



/* NAME:
 *   void MeasureTP508 (TP508_MEASDEP* pMeasSettings)
 * DESCRIPTION:
 *   This routine sets the TP508 card to perform one measurement on both
 *   channels
 * INPUT PARAMETERS:
 *   pMeasSettings   instrument and measurement settings
 * OUTPUT PARAMETERS:
 *   -
 */
void MeasureTP508 (TP508_MEASDEP* pMeasSettings)
{
 SetTrigLevel( pMeasSettings->TrigSlope,
                pMeasSettings->TrigLevel,
	        pMeasSettings->TrigHysteresis
             );
 SetTimeBase( &(pMeasSettings->SampleFreq) );
 SetPostCount( pMeasSettings->NumPostSamples );

 outportb( Ad_PortAddress+ContReg[1], (unsigned char)4 );
 outportb( Ad_PortAddress+DataReg[1], (unsigned char)0 ); // Clck Channel 1 normal
 outportb( Ad_PortAddress+ContReg[2], (unsigned char)4 );
 outportb( Ad_PortAddress+DataReg[2], (unsigned char)0 ); // Clck Channel 2 normal

/*
 * Reset Post Counter and Address Counter
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)4 );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0 );

/*
 * Set Clck Source
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)3 );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0 );

/*
 * Set Trig Source
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)7 );
 switch (pMeasSettings->TrigSource)
 {
  case CH1:
       outportb( Ad_PortAddress+DataReg[0], (unsigned char)0 );
       break;
  case CH2:
       outportb( Ad_PortAddress+DataReg[0], (unsigned char)1 );
       break;
  case NO_TRIG:
       outportb( Ad_PortAddress+DataReg[0], (unsigned char)2 );
       break;
  case CH1ORCH2:
       outportb( Ad_PortAddress+DataReg[0], (unsigned char)3 );
       break;
  case CH1ANDCH2:
       outportb( Ad_PortAddress+DataReg[0], (unsigned char)4 );
       break;
  case EXT:
       outportb( Ad_PortAddress+DataReg[0], (unsigned char)5 );
       break;
 }

/*
 * Reset Post Counter and Address Counter
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)4 );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)0 );

/*
 * Reset Trig Channel 1
 */
 outportb( Ad_PortAddress+ContReg[1], (unsigned char)5 );
 outportb( Ad_PortAddress+DataReg[1], (unsigned char)0 );

/*
 * Reset Trig Channel 2
 */
 outportb( Ad_PortAddress+ContReg[2], (unsigned char)5 );
 outportb( Ad_PortAddress+DataReg[2], (unsigned char)0 );

/*
 * Set Clck SRAM
 */
 outportb( Ad_PortAddress+ContReg[0], (unsigned char)3 );
 outportb( Ad_PortAddress+DataReg[0], (unsigned char)15 );

} //MeasureTP508



/* NAME:
 *   void DetermineOffset (void)
 * DESCRIPTION:
 *   This routine determines the offset in the channels, used to calibrate
 *   the channels
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void DetermineOffset (void)
{
 #define NUMOFFSET      50
 const int DEVIATION=20*256;
 #define OFFSET_W       160

 int           nI,
               nJ,
               nChannelIndex,
               nStand;
 long int      lHelpValue,
               PGAStand[NUMCHANNELS][6];
 TP508_MEASDEP MeasSettings;
 float         fOffsetHelp[NUMCHANNELS],
               fTimeDivValue;
 long          lHighValue = ((long)OFFSET_W*256*NUMOFFSET) + DEVIATION;
 long          lLowValue = ((long)OFFSET_W*256*NUMOFFSET) - DEVIATION;

 fOffsetHelp[0] = Offset_AD[0];
 fOffsetHelp[1] = Offset_AD[1];
 Offset_AD[0]   = -2.0 * ((float)OFFSET_W/256);
 Offset_AD[1]   = -2.0 * ((float)OFFSET_W/256);

 FillPGAStands (PGAStand);

 for (nChannelIndex = 0; nChannelIndex < NUMCHANNELS; nChannelIndex++)
 {
   for (nStand = 0; nStand <= 5; nStand++)
   {
     SetReferenceVoltageAD( Ref_Value[0][nStand], Ref_Value[1][nStand] );
     SetPGA( TRUE, TRUE, PGAStand[0][nStand]+PGAStand [1][nStand]+REF_ON );
     do
     {
       switch (nChannelIndex)
       {
         case CHANNEL1:
              AdjustOffsetDA( 0, DA_Value[0][nStand]+Offset_AD[0] );
              AdjustOffsetDA( 1, 0);
              break;
         case CHANNEL2:
              AdjustOffsetDA( 0, 0);
              AdjustOffsetDA( 1, DA_Value[1][nStand]+Offset_AD[1] );
              break;
       }

      /*
       * Get TimeDiv-Value
       */
       fTimeDivValue = DeriveFloatingPointNumber(TimeDivBuffer [3] );
       MeasSettings.SampleFreq      = fTimeDivValue;
       MeasSettings.NumPostSamples  = NUMOFFSET + 40;
       MeasSettings.NumPreSamples   = NUMOFFSET + 40;
       MeasSettings.TrigHysteresis  = 100;
       MeasSettings.TrigSlope       = POS;
       MeasSettings.TrigSource      = NO_TRIG;
       MeasSettings.MeasurementMode = ALT;

       MeasureTP508 (&MeasSettings);
       nJ = 0;
       do
       {
         for (nI = 1; nI <= 10; nI++)
           nJ++;
       }
       while (!ReadEOCBit ());
       ReadSRAMData (&MeasSettings);
       lHelpValue = DetermineMean( nChannelIndex,
                                   nStand,
                                   DEVIATION,
                                   MeasSettings.NumPreSamples
                                 );
     }
     while ( (lHelpValue > lHighValue) || (lHelpValue < lLowValue) );
   }
 }
 for ( nChannelIndex = 0; nChannelIndex < NUMCHANNELS; nChannelIndex++ )
 {
   nJ = 3;
   for (nI = 6; nI <= 8; nI++)
   {
     DA_Value[nChannelIndex][nI] = DA_Value[nChannelIndex][nJ];
     nJ++;
   }
   nJ = 3;
   for (nI = 9; nI < 12; nI++)
   {
     DA_Value[nChannelIndex][nI] = DA_Value[nChannelIndex][nJ];
     nJ++;
   }
 }
 Offset_AD[0] = fOffsetHelp [0];
 Offset_AD[1] = fOffsetHelp [1];

}//DetermineOffset



/* NAME:
 *   long int DetermineMean( int nChannelIndex,
 *                           int nStand,
 *                           int nDeviation,
 *                           int nBeginValue
 *                         )
 * DESCRIPTION:
 *   This routine calculates the average value of NUMOFFSET samples in a
 *   waveform record, used to determine the offset in a channel
 * INPUT PARAMETERS:
 *   nChannelIndex  indicating which channel to use
 *   nStand         the V/div setting to use
 *   nDeviation
 *   nBeginValue    start position in the waveform record
 * OUTPUT PARAMETERS:
 *   returnvalue    the mean value
 */
long int DetermineMean( int nChannelIndex,
                        int nStand,
                        int nDeviation,
                        int nBeginValue
                      )
{
 long int lHelpValue = 0, lLowValue, lHighValue;
 int nI;
 unsigned int far* pRAMData;

 switch (nChannelIndex)
 {
   case CHANNEL1:
        pRAMData = pGlobalRAMDataCH1;
        break;
   case CHANNEL2:
        pRAMData = pGlobalRAMDataCH2;
        break;
 }
 for ( nI = nBeginValue; nI < nBeginValue+NUMOFFSET; nI++ )
 {
   lHelpValue += *(pRAMData + nI);
 }
 lHighValue = (long)OFFSET_W*256*NUMOFFSET + nDeviation;
 lLowValue  = (long)OFFSET_W*256*NUMOFFSET - nDeviation;

 if (lHelpValue > lHighValue)
 {
   DA_Value[nChannelIndex][nStand] += (float)5/2048;
 }

 if (lHelpValue < lLowValue)
 {
   DA_Value[nChannelIndex][nStand] -= (float)5/2048;
 }

 return( lHelpValue );

} // DetermineMean



/* NAME:
 *   void FillPGAStands( long int PGAStand[NUMCHANNELS+1][6] )
 * DESCRIPTION:
 *   This routine fills the PGA settings
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   PGAStand[][]  array containing the PGA settings
 */
void FillPGAStands (long int PGAStand [NUMCHANNELS + 1][6])
{

 PGAStand[0][0] = PGA1_GAIN_1_10 + PGA1_GAIN_4;
 PGAStand[0][1] = PGA1_GAIN_1_10 + PGA1_GAIN_2;
 PGAStand[0][2] = PGA1_GAIN_1_10 + PGA1_GAIN_1;
 PGAStand[0][3] = PGA1_GAIN_4;
 PGAStand[0][4] = PGA1_GAIN_2;
 PGAStand[0][5] = PGA1_GAIN_1;

 PGAStand[1][0] = PGA2_GAIN_1_10 + PGA2_GAIN_4 + PGA2_ON;
 PGAStand[1][1] = PGA2_GAIN_1_10 + PGA2_GAIN_2 + PGA2_ON;
 PGAStand[1][2] = PGA2_GAIN_1_10 + PGA2_GAIN_1 + PGA2_ON;
 PGAStand[1][3] = PGA2_GAIN_4 + PGA2_ON;
 PGAStand[1][4] = PGA2_GAIN_2 + PGA2_ON;
 PGAStand[1][5] = PGA2_GAIN_1 + PGA2_ON;

} //FillPGAStands


/* NAME:
 *   void DetermineGain(void)
 * DESCRIPTION:
 *   This routine determines the gain for the channel, for calibrating the
 *   chanels
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   -
 */
void DetermineGain (void)
{
 int nChannelIndex, nI, nJ;
 long int PGAStand [NUMCHANNELS + 1] [6];

 FillPGAStands (PGAStand);

 DetermineGainCHX (CHANNEL1, PGAStand);
 DetermineGainCHX (CHANNEL2, PGAStand);

 for ( nChannelIndex = 0; nChannelIndex < NUMCHANNELS; nChannelIndex++ )
 {
  nJ = 3;
  for (nI = 6; nI <= 8; nI++)
  {
   Ref_Value[nChannelIndex][nI] = Ref_Value[nChannelIndex][nJ];
   nJ++;
  }
  nJ = 3;
  for (nI = 9; nI < 12; nI++)
  {
   Ref_Value[nChannelIndex][nI] = Ref_Value[nChannelIndex][nJ];
   nJ++;
  }
 }
} //DetermineGain



/* NAME:
 *   void DetermineGainCHX( int nChannelIndex,
 *                          long int PGAStand[NUMCHANNELS+1][6]
 *                        )
 * DESCRIPTION:
 *   This routine determines the gain for a channel
 * INPUT PARAMETERS:
 *   nChannelIndex   indicates the channel to calibrate
 *   PGAStand[][]    the pga settings
 * OUTPUT PARAMETERS:
 *   -
 */
void DetermineGainCHX( int nChannelIndex, long int PGAStand[NUMCHANNELS+1][6] )
{
 #define NUMGAIN        100
 #define DEVIATION      20*256
 float fMAX_UITSTURING=(float)250/256;
 #define RESISTOR_ATTENUATION 1;

 unsigned long lHelpValue, lMeasValue;
 TP508_MEASDEP MeasSettings;
 int nMean, nStand;
 float fCalValue, fHelpValue, fVoltDivVal;
 BOOL bRefValSmal255 = FALSE;

 for (nStand = 0; nStand <= 5; nStand++)
 {
   fVoltDivVal = DeriveFloatingPointNumber (VoltDivBuffer[nStand])*4;
   fCalValue = fVoltDivVal*fMAX_UITSTURING*25*RESISTOR_ATTENUATION;
   if (fCalValue > 4.96093)
   {
     fCalValue = 4.96093;
   }
   fHelpValue =  fCalValue*32767*0.04;
   lMeasValue = (32767 + lRound((double)(fHelpValue/fVoltDivVal)))*NUMGAIN;
   switch (nChannelIndex)
   {
     case CHANNEL1:
          AdjustOffsetDA( 0, (Offset_AD[0] + DA_Value[0][nStand]) );
          AdjustOffsetDA( 1, fCalValue );
          break;
     case CHANNEL2:
          AdjustOffsetDA( 0, fCalValue );
          AdjustOffsetDA( 1, Offset_AD[1] + DA_Value[1][nStand] );
          break;
   }
   SetPGA( TRUE, TRUE, PGAStand[0][nStand] + PGAStand[1][nStand] + REF_ON );
   nMean = 0;
   do
   {
     nMean++;
     SetReferenceVoltageAD( Ref_Value[0][nStand], Ref_Value[1][nStand] );
    /*
     * Wait 10msec
     */
     delay (10);
     MeasSettings.SampleFreq      = 5e6;
     MeasSettings.NumPostSamples  = NUMGAIN + 40;
     MeasSettings.NumPreSamples   = NUMGAIN + 40;
     MeasSettings.TrigHysteresis  = 0;
     MeasSettings.TrigSlope       = POS;
     MeasSettings.TrigSource      = NO_TRIG;
     MeasSettings.MeasurementMode = ALT;
     MeasureTP508( &MeasSettings );
     do
     {
       // do nothing, just wait
     }
     while (!ReadEOCBit ());
     ReadSRAMData (&MeasSettings);
     lHelpValue = CheckMeanMeasurementData( lMeasValue,
                                            nStand,
                                            DEVIATION,
                                            NUMGAIN,
                                            nChannelIndex,
                                            40 + MeasSettings.NumPreSamples
                                          );

     if (Ref_Value[nChannelIndex][nStand] < 255)
     {
       bRefValSmal255 = TRUE;
     }
   }
   while ( ( (lHelpValue > lMeasValue + DEVIATION) ||
             (lHelpValue < lMeasValue - DEVIATION)
           ) && (bRefValSmal255) && (nMean < 128)
         );
  }

} // DetermineGainCHX



/* NAME:
 *   unsigned long CheckMeanMeasurementData ( unsigned long lMeasValue,
 *                                            int nStand,
 *                                            int nDeviation,
 *                                            int nNumGain,
 *                                            int nChannelIndex,
 *                                            int nBeginValue
 *                                           )
 * DESCRIPTION:
 *   This routine checks the mean measurement data
 * INPUT PARAMETERS:
 *   lMeasValue    the preferred value
 *   nStand        setting to check
 *   nDeviation    the allowable deviation
 *   NumGain       number of samples to check
 *   ChannelIndex  which channel to check
 *   BeginValue    point in waveform record to start
 * OUTPUT PARAMETERS:
 *   returnvalue   the found value
 */
unsigned long CheckMeanMeasurementData ( unsigned long lMeasValue,
                                         int nStand,
                                         int nDeviation,
                                         int nNumGain,
                                         int nChannelIndex,
                                         int nBeginValue
                                       )
{
 unsigned int far* pRAMData;
 unsigned long lHelpValue = 0;
 int nCount;

 switch (nChannelIndex)
 {
  case CHANNEL1:
       pRAMData = pGlobalRAMDataCH1;
       break;
  case CHANNEL2:
       pRAMData = pGlobalRAMDataCH2;
       break;
 }

 for ( nCount = nBeginValue; nCount <= nNumGain+nBeginValue; nCount++ )
 {
   lHelpValue+= *(pRAMData + nCount);
 }

 if (lHelpValue > (lMeasValue + nDeviation))
 {
   (Ref_Value[nChannelIndex][nStand])++;
 }

 if (lHelpValue < (lMeasValue - nDeviation))
 {
   (Ref_Value[nChannelIndex][nStand])--;
 }

 return (lHelpValue);

} // CheckMeanMeasurementData



#ifdef DEMO
/* NAME:
 *   void Calibrate (BOOL* pbCalError)
 * DESCRIPTION:
 *   This is a dummy routine for calibrating the TP508, used when compiling
 *   this file with a compiler directive DEMO on. The routine just waits.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pbCalError   flag to indicate the calibration result, is set to FALSE
 *                to indicate 'no error' to calling function
 */
void Calibrate (BOOL* pbCalError)
{
 /*
  * Wait 1000msec
  */
  delay ( 1000 );
  *pbCalError = FALSE;
}
#else
/* NAME:
 *   void Calibrate (BOOL* pbCalError)
 * DESCRIPTION:
 *   This routine is used when compiled without the compiler directive DEMO.
 *   The routine calibrates the TP508 card.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pbCalError   flag to indicate the calibration result
 */
void Calibrate (BOOL* pbCalError)
{
  AdjustOffsetDA (0, DA_Value [0][0] + Offset_AD[0]);
  AdjustOffsetDA (1, DA_Value [0][1] + Offset_AD[1]);

 /*
  * Offset <-----> Gain (Apply Convergence-Method)
  */
  CalibrateOnce (pbCalError);

 /*
  * Try to adjust Offset and Gain again
  */
  if (*pbCalError)
    CalibrateOnce (pbCalError);

  DA_Value[0][0] = DA_Value[0][0] - (float)40/2048;
  DA_Value[1][0] = DA_Value[1][0] - (float)40/2048;
  DA_Value[0][1] = DA_Value[0][1] - (float)10/2048;
  DA_Value[1][1] = DA_Value[1][1] - (float)10/2048;

} // Calibrate
#endif


/* NAME:
 *   void CalibrateOnce (BOOL* pbCalError)
 * DESCRIPTION:
 *   This routine calibrates the TP508 once
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pbCalError   flag to indicate the calibration result
 */
void CalibrateOnce (BOOL* pbCalError)
{
  int nI, nChannelIndex, nStand;

  for (nI = 1; nI <= 2; nI++)
  {
   /*
    * DetermineOffset ();
    */
    DetermineGain ();
  }
  DetermineOffset (); // Could have been changed in DetermineGain ()
 /*
  * Reset CalError
  */
  *pbCalError = FALSE;

 /*
  * Check all reference values, if one is out of range, set the error flag
  */
  for ( nChannelIndex = 0; nChannelIndex < NUMCHANNELS; nChannelIndex++ )
    for ( nStand = 0; nStand < MAXVOLTDIVSTAND; nStand++ )
      if ( (Ref_Value [nChannelIndex][nStand] < 10)  ||
           (Ref_Value [nChannelIndex][nStand] > 250)
         )
        {
          /*
           * this one is out of range
           */
           *pbCalError = TRUE;
        }

} // CalibrateOnce


/* NAME:
 *   BOOL Calibrate_TP508( void )
 * DESCRIPTION:
 *   this routine calls the calibration function
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   returnvalue  TRUE   calibration successfull
 *                FALSE  calibration failed
 */
BOOL Calibrate_TP508 (void)
{
  BOOL bCalError;

  Calibrate( &bCalError );
  if (bCalError)
    return(FALSE);
  else

  return(TRUE);

} // Calibrate_TP508



