{N+} { use Floating point coprocessor }

program Demo_Scope;
(*
 * This file contains a demo scope program, which can be used
 * for the TP112, TP208, the TP508 or the HS508.
 *
 * To make a program for a specific device, compile this file
 * with a conditional compiler directive.
 * Possible values for this compiler directive are:
 *
 * TP112         { to compile for a TP112 }
 * TP208         { to compile for a TP208 }
 * TP508         { to compile for a TP508 }
 * HS508         { to compile for a HS508 }
 *
 *)


{$G+}

uses dos,
     crt,
     graph;

{$I vars}

{$IfDef TP112          }
{$I TP112\HARDWARE.PAS }   { You are making a TP112 program }
{$EndIf                }

{$IfDef TP208          }
{$I TP208\HARDWARE.PAS }   { You are making a TP208 program }
{$EndIf                }

{$IfDef TP508          }
{$I TP508\HARDWARE.PAS }   { You are making a TP508 program }
{$EndIf                }

{$IfDef HS508          }
{$I HS508\HARDWARE.PAS }   { You are making a HS508 program }
{$EndIf                }


const Ch1Color   = Yellow;
      Ch2Color   = lightCyan;
      GridColor  = Brown;
      BackColor  = Black;
      Vectors    = True;
      Dots       = False;

var   OldWave    : MeasurementDataType; { contains last displayed data      }
      GM, GD     : integer;             { Graphics Mode and Graphics Driver }
      Chr        : Char;                { to empty keyboard buffer          }
      rCalSignal : real;                { frequency for generator output    }


procedure DrawSignal( Ch : integer; DrawLines : boolean );
(*
 * DESCRIPTION
 *   This routine draws the signal of one channel
 * INPUT
 *   Ch        : indicates which channel to display
 *   DrawLines : True  : draw using vectors
 *               False : draw using dots
 * OUTPUT
 *   -
 *)
var i            : integer; { loop counter                         }
    DrawColor    : integer; { color to draw the signal with        }
    XScaleFactor,           { scaling factor for X direction       }
    YScaleFactor : real;    { scaling factor for Y direction       }
    MaxY         : integer; { Screen height in pixels              }
begin

 (*
  * first determine which color to draw with
  *)
  if   Ch = Ch1
  then DrawColor := Ch1Color
  else DrawColor := Ch2Color;

  with MeasureSetting do
  begin
   (*
    * Determine the X and Y scaling factors.
    * Use the entire screen of the PC to draw the signal on.
    *)
    XscaleFactor := GetMaxX / (MeasureSetting.lPostSamples + Measuresetting.lPreSamples);
    YscaleFactor := GetMaxY / DeviceMaxValue;
    MaxY         := GetMaxY;

   (*
    * Now first draw the grid: a horizontal line through zero and a vertical
    * line through the trigger point
    *)
    setcolor( GridColor );
    Line( round(lPreSamples * XScaleFactor),
          0,
          round(lPreSamples * XScaleFactor),
          GetMaxY
        );
    Line( 0,
          round( GetMaxY/2 ),
          GetMaxX,
          round( GetMaxY/2 )
        );


   (*
    * Now go draw the signal
    * This is done one sample at a time, to avoid flashing traces
    * and to increase speed
    *)

    i := -lPreSamples;
    repeat
     (*
      * First remove the previous signal
      *)
      SetColor( BackColor );
      if   DrawLines
      then begin
             MoveTo(      round( XScaleFactor*(i+lPreSamples  ) ),
                     MaxY-trunc( YscaleFactor*(OldWave[ch][i  ]) )
                   );
             LineTo(      round( XScaleFactor*(i+lPreSamples+1) ),
                     MaxY-trunc( YscaleFactor*(OldWave[ch][i+1]) )
                   );
           end
      else begin
             PutPixel(      round( XScaleFactor*(i+lPreSamples ) ),
                       MaxY-trunc( YscaleFactor*(OldWave[ch][i]) ),
                       BackColor
                     )
           end;

     (*
      * part of old trace is removed, now draw part of new trace
      *)
      setColor( DrawColor );
      if   DrawLines
      then begin
             MoveTo(      round( XScaleFactor*(i+lPreSamples  ) ),
                     MaxY-trunc( YscaleFactor*(MeasureData[ch][i  ]) )
                   );
             LineTo(      round( XScaleFactor*(i+lPreSamples+1) ),
                     MaxY-trunc( YscaleFactor*(MeasureData[ch][i+1]) )
                   );
           end
      else begin
             PutPixel(      round( XScaleFactor*(i+lPreSamples     ) ),
                       MaxY-trunc( YscaleFactor*(MeasureData[ch][i]) ),
                       DrawColor
                     );
           end;

     (*
      * now copy new sample to array containing previous signal
      *)
      OldWave[Ch][i] := MeasureData[Ch][i];

     (*
      * goto next sample
      *)
      inc(i)

    until i = lPostSamples-1; { not  until (i= lPostSamples) while line is }
                              { drawn from Data[Ch][i] to Data[Ch][i+1]    }

   (*
    * due to the stop criterium, the last sample is not copied, do it now
    *)
    OldWave[Ch][i] := MeasureData[Ch][i];

  end; { with MeasureSettings do }

end; { DrawSignal }


begin

  clrscr;

 (*
  * Test for hardware present,
  * calibrate instrument
  *)
  writeln( strText0 );
  InitInstrument;

  if   HardwarePresent
  then begin
        (*
         * Open graphics display
         *)
         GD := detect;
         initgraph( GD, GM, '');

         OutTextXY( 1,  1, DeviceString );
         SetColor ( Ch1Color );
         OuttextXY( 1, 12, 'Ch1' );
         SetColor ( Ch2Color );
         OuttextXY( 1, 24, 'Ch2' );

         with MeasureSetting do
         begin
           bySwitch[Ch1]    := Pga20Volt;{MaxSetting shr 1;{ Pga20mVolt, Pga40mVolt..Pga80Volt}
           bySwitch[Ch2]    := Pga20Volt;{ Pga20mVolt, Pga40mVolt..Pga80Volt}
           bCoupling[Ch1]   := Ac;       { CoupleDc, CoupleAc                }
           bCoupling[Ch2]   := Ac;       { CoupleDc, CoupleAc                }
           nTrigSource      := Ch1Trig;  { Ch1Trig, Ch2Trig, NoTrig, ExtTrig }
           byTrigLevel[Ch1] := 127;      { 0 .. 255                          }
           byTrigLevel[Ch2] := 127;      { 0 .. 255                          }
           byTrigHys        := 10;       { 0 .. 255                          }
           bTrigSlope       := PosSlope; { PosSlope, NegSlope                }
           lPreSamples      := round(0.15*GetMaxX); {0 .. MaxPreSamples      }
           lPostSamples     := GetMaxX-lPreSamples; {0 .. MaxPostSamples     }
           rDcLevel[ch1]    := 0;        { -4 .. +4                          }
           rDcLevel[ch2]    := 0;        { -4 .. +4                          }
           rSampleFreq      := MaxSamplingFreq; {1e-3 .. device dependant    }
           byMeasureMode    := Chop;     { Ch1, Ch2, Cmp, Chop               }
           lTimeOut         := 3;       { 0 .. 2147483647                   }
                                         { TimeOut=10 means:                 }
                                         { wait (10/18.2) sec for triggering }
                                         { If triggering not occurs after    }
                                         { 0.55 sec, start measuring         }
         end; { with }

         rCalSignal := 1e+4;
         SetCalOutput( rCalSignal );

         repeat
          (*
           * Perform one measurement
           *)
           StartMeasurement( MeasureSetting, MeasureData );

          (*
           * display the measured values
           *)
           DrawSignal( Ch1, Vectors );
           DrawSignal( Ch2, Vectors );

         until keypressed;

        (*
         * a key is pressed, empty keyboard buffer
         *)
         Chr := ReadKey;
         if   Chr =  #0
         then Chr := ReadKey; {function key pressed, read second value }

         closegraph;

       end; { if HardwarePresent }

end.