(*
 * This file contains TP208-specific routines.
 * It is to be included in the main source file
 *)


const
      LPT1Address      = $378;
      LPT2Address      = $278;
      LPT3Address      = $3BC;
      TP208Address     = $300;
      TP508Address     = $300;
      TP112Address     = $300;

      ContReg          : array[1..3] of Byte = ( 0, 4, 6 );
      DataReg          : array[1..3] of Byte = ( 1, 5, 7 );

      PostLoCon        = 0;
      PostHiCon        = 1;
      TimebaseCon      = 2;
      ModeCon          = 3;
      ResetCon         = 4;
      CountCon         = 5;
      WriteTimebaseCon = 6;

      Level1ConAd      : array[1..2] of Byte = ( 0, 0 );
      Level2ConAd      : array[1..2] of Byte = ( 1, 1 );
      TrigLevelConAd   : array[1..2] of Byte = ( 2, 2 );
      PGAConAd         : array[1..2] of Byte = ( 3, 3 );
      DateZeroConAd    : array[1..2] of Byte = ( 4, 4 );
      IRQConAd         : array[1..2] of Byte = ( 5, 5 );
      TrigResetConAd   : array[1..2] of Byte = ( 6, 6 );

      Level1RegAd      : array[1..2] of Byte = ( 0, 0 );
      Level2RegAd      : array[1..2] of Byte = ( 0, 0 );
      TrigLevRegAd     : array[1..2] of Byte = ( 0, 0 );
      PgaRegAd         : array[1..2] of Byte = ( 0, 0 );
      TrigResetRegAd   : array[1..2] of Byte = ( 0, 0 );

      PostLoReg        : byte = 0;
      PostHiReg        : byte = 0;
      OutputReg        : byte = 0;
      TimebaseReg      : byte = 0;
      ModeReg          : byte = 0;
      ResetReg         : byte = 0;
      ContrReg         : byte = 0;
      WriteTimebaseReg : byte = 0;

      PGASwitchTime    = 10;


const Pga20mVolt  =  1;
      Pga40mVolt  =  2;
      Pga80mVolt  =  3;
      Pga200mVolt =  4;
      Pga400mVolt =  5;
      Pga800mVolt =  6;
      Pga2Volt    =  7;
      Pga4Volt    =  8;
      Pga8Volt    =  9;
      Pga20Volt   = 10;
      Pga40Volt   = 11;
      Pga80Volt   = 12;

      Pga1Gain2   =      1;
      Pga1Gain1   =      2;
      Pga1Gain10  =      4;
      Pga2Gain10  =      8;
      Pga2Gain1   =     16;
      Pga2Gain2   =     32;
      Pga2Gain4   =     64;
      Pga2on      =    128;
      Pga1on      =    256;
      Pga1Gain4   =    512;
      Pga1Att10   =   1024;
      Pga1Att1    =   2048;
      Pga1AcDc    =   4096;
      Pga2AcDc    =   8192;
      Pga2Att1    =  16384;
      Pga2Att10   =  32768;
      Pga2Att100  =  65536;
    { NotUsed     = 131072; }
      RefOn       = 262144;
      Pga1Att100  = 524288;

      MaxSetting = 12;      { Hightest input range setting available }
      MinSetting =  1;      { Lowest   input range setting available }

      CristalFreq     = 20e+6;
      MaxSamplingFreq = 20e+6;
      DeviceMaxValue  = 255;  { 8 bits instrument }

      DeviceString    = 'TP208';

{$F+}  { force FAR function calls }
procedure TimeOutTimer;
(*
 * DESCRIPTION:
 *   This interrupt routine is called 18.2 times per second. It is used to
 *   to determine whether the trigger system is timed out and a start of
 *   measurement is to be forced
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
interrupt;
begin
  inc( lTimerCount );
end;{TimeOutTimer}


procedure ProgramExit;
(*
 * DESCRIPTION:
 *   This routine is called when the program is to be terminated.
 *   It brings the computer back to its normal state.
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
begin
 (*
  * Restore normal exit routine
  *)
  ExitProc := ExitSave;

 (*
  * Restore normal timer interrupt routine
  *)
  setintvec($1c,Int1cSave);

 (*
  * Put the TP208 card to the normal state
  *)
  port[DeviceAddress+4]:=3;
  delay(PgaSwitchTime);
  port[DeviceAddress+5]:=0;
end;{ ProgramExit }
{$F-}  { use NEAR function calls }

procedure SetRef( rValue1, rValue2 : Real );
(*
 * DESCRIPTION:
 *   This routine sets the reference voltages for the ADC's of the TP208
 *   The values have to be written serial to the DAC.
 * INPUT:
 *   rValue1     : the value for Ch1
 *   rValue2     : the value for Ch2
 * OUTPUT:
 *   -
 *)
const PgaBlnk          = 64;
      RefSlct          = 32;
      SerialDataLength =  7;  { 8 bits to send   }
var   bySclck : Byte;         { Serial clock bit }
      bySdata : Byte;         { serial data bit  }
      wHelp   : Word;
      Index   : Integer;
      nValue1,
      nValue2 : Integer;
begin

 (*
  * First cast the input parameters to integers
  *)
  nValue1 := round( rValue1 );
  nValue2 := round( rValue2 );

 (*
  Setup the serial communication
  *)
  port[DeviceAddress+4] := 3;
  wHelp                := round(exp(SerialDataLength*ln(2))); {128}
  bySdata              := 0;
  bySclck              := 0;
  port[DeviceAddress+5] := bySdata + bySclck + PgaBlnk;
  port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;
  bySclck              := 2;
  port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;

 (*
  * send Value1
  *)
  for Index := 0 to SerialDataLength do
  begin
    if   (nValue1 and wHelp) = 0
    then bySdata := 0
    else bySdata := 1;

    bySclck              := 0;
    port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;
    bySclck              := 2;
    port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;
    wHelp                := wHelp div 2;

  end;

 (*
  * Send Value2
  *)
  wHelp := round(exp(SerialDataLength*ln(2))); {128}
  for Index := 0 to SerialDataLength do
  begin
    if   (nValue2 and wHelp) = 0
    then bySdata := 0
    else bySdata := 1;

    bySclck              := 0;
    port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;
    bySclck              := 2;
    port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;
    wHelp                := wHelp div 2;
  end;

 (*
  * Close the serial communication
  *)
  bySdata              := 0;
  bySclck              := 0;
  port[DeviceAddress+5] := bySdata + bySclck + RefSlct + PgaBlnk;
  port[DeviceAddress+5] := PgaBlnk;

end; { SetRef }


procedure SetPga( bCouplingCh1AC, bCouplingCh2AC : Boolean; lValue : Longint );
(*
 * DESCRIPTION:
 *   This routine sets the coupling switches for Ch1 and Ch2 and the
 *   attenuators for both Ch1 and Ch2.
 *   The data is sent serial
 * INPUT:
 *   bCouplingCh1AC : the requested state for the Ch1 coupling switch
 *   bCouplingCh2AC : the requested state for the Ch2 coupling switch
 *   lValue         : a combined parameter for the attenuator switches
 * OUTPUT:
 *  -
 *)
const PgaBlnk          = 64;
      PgaSlct          = 16;
      SerialDataLength = 19;   { 20 bits to send  }
var   bySclck,                 { Serial clock bit }
      bySdata : Byte;          { Serial data bit  }
      lHelp   : Longint;
      Index   : Integer;
begin
 (*
  * Add the coupling switch settings to Value
  *)
  if   not bCouplingCh1AC
  then lValue := lValue + Pga1AcDc;
  if   not bCouplingCh2AC
  then lValue := lValue + Pga2AcDc;

 (*
  * init the serial communication
  *)
  port[DeviceAddress+4] := 3;
  lHelp                := round(exp(SerialDataLength*ln(2)));  {1048576}
  bySclck              := 0;
  bySdata              := 0;
  port[DeviceAddress+5] := bySdata + bySclck + PgaBlnk;

 (*
  * send the 20 bits to the switch driver circuitry
  *)
  for Index := 0 to SerialDataLength do
  begin
    if   (lValue and lHelp) = 0
    then bySdata := 0
    else bySdata := 1;
    bySclck              := 0;
    port[DeviceAddress+5] := bySdata+bySclck+PgaBlnk;
    bySclck              := 2;
    port[DeviceAddress+5] := bySdata+bySclck+PgaBlnk;
    lHelp                := lHelp div 2;
  end;

 (*
  * Back to normal state
  *)
  port[DeviceAddress+5] := PgaSlct+PgaBlnk;
  port[DeviceAddress+5] := PgaBlnk;
  delay(PgaSwitchTime);

end; { SetPga }


procedure SetDa( nCh : Integer; rValue : Real );
(*
 * DESCRIPTION:
 *   This routine sets the DC offset level for a specified channel
 *   The value has to be sent serial
 * INPUT:
 *   nCh    : indicates which channel to change
 *   rValue : the new value for the DAC
 * OUTPUT:
 *   -
 *)
const PgaBlnk          = 64;
      AdSlct : array[1..2] of Byte = (4,8);
      TopScrMin1       = 4095;
      HalfScr          = 2048;
      BotScr           = 0;
      SerialDataLength = 15;    { 16 bits to send }
var   bySclck : Byte;
      bySdata : Byte;
      wHelp   : Word;
      Index   : Integer;
      nData   : Integer;
begin
 (*
  * Start init serial communication
  *)
  port[DeviceAddress+4] := 3;

 (*
  * Scale, shift, clip, round the Value and set bit 12
  *)
  rValue      := ( (rValue/5) * HalfScr ) + HalfScr;
  if   rValue >  TopScrMin1
  then rValue := TopScrMin1;
  if   rValue <  BotScr
  then rValue := BotScr;
  nData       := round(rValue);
  nData       := nData xor HalfScr;

 (*
  * Continue init serial communication
  *)
  wHelp   := round(exp(SerialDataLength*ln(2)));  {32768}
  bySclck := 2;
  bySdata := 0;
  port[DeviceAddress+5] := bySdata + bySclck + AdSlct[nCh] + PgaBlnk;

 (*
  * send all 16 bits
  *)
  for Index := 0 to SerialDataLength do
  begin
    if   (nData and wHelp) = 0
    then bySdata := 0
    else bySdata := 1;
    bySclck              := 2;
    port[DeviceAddress+5] := bySdata + bySclck + AdSlct[nCh] + PgaBlnk;
    bySclck              := 0;
    port[DeviceAddress+5] := bySdata + bySclck + AdSlct[nCh] + PgaBlnk;
    wHelp                := wHelp div 2;
  end;

  port[DeviceAddress+5] := PgaBlnk;

end; { SetDA }


function EOC : Boolean;
(*
 * DESCRIPTION:
 *   This routine reads the End Of Conversion bit
 * INPUT:
 *   -
 * OUTPUT:
 *   returnvalue        : The state of the End Of Conversion bit
 *)
begin

  EOC := (port[DeviceAddress+2] and 1) = 1;

end; { EOC }


procedure SetPostCount( wPostCount : Word );
(*
 * DESCRIPTION:
 *   This procedure sets the Post counter to the requested value
 * INPUT:
 *   PostCount  : the requestd value for the postcounter
 * OUTPUT:
 *   -
 *)
var byPostLoReg : Byte;
    byPostHiReg : Byte;
begin

  byPostLoReg                   := lo( wPostCount );
  byPostHiReg                   := hi( wPostCount );
  port[DeviceAddress+ContReg[1]] := PostLoCon;
  port[DeviceAddress+DataReg[1]] := byPostLoReg;
  port[DeviceAddress+ContReg[1]] := PostHiCon;
  port[DeviceAddress+DataReg[1]] := byPostHiReg;

end; { SetPostCount }


procedure SetTriglevel( InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION:
 *   This routine sets the triggerlevel for both channels.
 * INPUT:
 *   InstrumentSettings : a structure holding some instrumentsettings
 * OUTPUT:
 *   -
 *)
var nCh  : Integer;
    nHelp : Integer;

  function nClip( nValue : integer ) : Integer;
 (*
  * nClip( nValue : integer ) : integer;
  *   A local routine that clips the integer input parameter nValue between
  *   0 and 255
  *)
  begin
    if   nValue >  255
    then nValue := 255;
    if   nValue <    0
    then nValue :=   0;
    nClip := nValue;
  end; { nClip }

begin
 (*
  * do both channels
  *)
  for nCh := Ch1 to Ch2 do
  begin
   (*
    * The trigger system has two triggerlevels, Level1 and Level2.
    * setup the two levels and the source
    *)
    with InstrumentSettings do
    if   bTrigSlope = PosSlope
    then begin
          (*
           * Level1 has to be 'hysteresis' lower than Level2
           *)
           Level1RegAd[nCh] := nClip( byTrigLevel[nCh]-byTrigHys );
           Level2RegAd[nCh] := nClip( byTrigLevel[nCh]           );
           if   byTrigSource = ExtTrig
           then TrigLevRegAd[nCh] := 0 {0001 0001}
           else TrigLevRegAd[nCh] := 5 {0000 0101}
         end
    else begin
          (*
           * Level1 has to be 'hysteresis' higher than Level2
           *)
           Level1RegAd[nCh] := nClip( byTrigLevel[nCh]+byTrigHys );
           Level2RegAd[nCh] := nClip( byTrigLevel[nCh]           );
           if   byTrigSource = ExtTrig
           then TrigLevRegAd[nCh] := 2 {0001 0000}
           else TrigLevRegAd[nCh] := 6 {0000 0110}
         end;

   (*
    * send the values to the trigger circuitry
    *)
    port[DeviceAddress+ContReg[nCh+1]] := TrigLevelConAd[nCh];
    port[DeviceAddress+DataReg[nCh+1]] := TrigLevRegAd[nCh];

    port[DeviceAddress+ContReg[nCh+1]] := Level1ConAd[nCh];
    port[DeviceAddress+DataReg[nCh+1]] := Level1RegAd[nCh];
    port[DeviceAddress+ContReg[nCh+1]] := Level2ConAd[nCh];
    port[DeviceAddress+DataReg[nCh+1]] := Level2RegAd[nCh]

  end

end; { SetTriggerLevel }


procedure SetTrigbit;
(*
 * DESCRIPTION:
 *   This routine sets the triggerbit, to force a start of measurement
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
begin

  port[DeviceAddress  ] := 7;
  port[DeviceAddress+1] := 2;

end; { SetTrigBit }


procedure SetTimebase( var rFreq : Real );
(*
 * DESCRIPTION:
 *   This routine sets the timebase of the TP208 to the requested sampling
 *   frequency
 * INPUT:
 *   rFreq      : the requested sampling frequency
 * OUTPUT:
 *   rFreq      : the set sampling frequency (input is clipped)
 * CAVEAT
 *   This routine has several exit points
 *)
const MaxFreq         = 20e+6;  { max sampling frequency TP208               }
      FreqDiv2        = 10e+6;  { sampling frequency / 2                     }
      FreqDiv5        =  4e+6;  { sampling frequency / 5                     }
      FreqDiv500      =  4e+4;  { sampling frequency / 500                   }
      FreqLoLim       =    62;  { lower limit frequency to be generated with }
                                { one 16-bits divider  (4e6/65535)           }
      MinFreq         =  1e-3;  { min allowed sampling frequency             }
      Div100          =   100;  { used in divide by 100                      }
var   wDivisionFactor : Word;
      rInputFreq      : Real;
      byTimebaseReg   : Byte;

begin
 (*
  * first clip the frequency to reasonable values
  *)
  if   rFreq >  MaxFreq
  then rFreq := MaxFreq;
  if   rFreq <  MinFreq
  then rFreq := MinFreq;

 (*
  * The three fastest timebase settings are made by dividing the
  * clock oscillator by 1, 2 or 5
  *)
  if   rFreq = MaxFreq
  then begin
         byTimebaseReg                 := 0+32;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         exit
       end;

  if   rFreq = FreqDiv2
  then begin
         byTimebaseReg                 := 4+32;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         exit
       end;

  if   rFreq = FreqDiv5
  then begin
         byTimebaseReg                 := 8+32;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         exit
       end;

 (*
  * the rest of the frequencies are generated by a timer chip
  *)

  rInputFreq := FreqDiv5;

  if   rFreq >  FreqDiv5
  then rFreq := FreqDiv5;

  if   rFreq <  FreqLoLim
  then rInputFreq := FreqDiv500;

  wDivisionFactor := round(rInputFreq/rFreq);
  rFreq           := rInputFreq/wDivisionFactor;

  byTimebaseReg                 := 3+32;
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;
  port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
  port[DeviceAddress+DataReg[1]] := 54; { 00 11 011 0 }
     (*
      *  00  [ = select counter 0                   ]
      *  11  [ = write least significant Byte first ]
      *  011 [ = mode 3, divide by n counter        ]
      *  0   [ = 16-bits counter                    ]
      *)
  byTimebaseReg                 := 0+32;
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;
  port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
  port[DeviceAddress+DataReg[1]] := lo(wDivisionFactor);
  port[DeviceAddress+DataReg[1]] := hi(wDivisionFactor);

  if   rFreq < FreqLoLim
  then begin
        (*
         * extra divider is needed
         *)
         byTimebaseReg                 := 3+32;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
         port[DeviceAddress+DataReg[1]] := 118; {01 11 011 0}
            (*
             * 01  [ = select counter 1                   ]
             * 11  [ = write least significant byte first ]
             * 011 [ = mode 3, divide by n counter        ]
             * 0   [ = 16-bits counter                    ]
             *)
         byTimebaseReg                 := 1+32;
         port[DeviceAddress+ContReg[1]] := TimebaseCon;
         port[DeviceAddress+DataReg[1]] := byTimebaseReg;
         port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
         port[DeviceAddress+DataReg[1]] := lo(Div100);
         port[DeviceAddress+DataReg[1]] := hi(Div100);

        (*
         * select clck input b
         *)
         byTimebaseReg := 16+32
       end
  else begin
        (*
         * can be made using 1 divider, so no extra divider needed
         *
         * select clck input a
         *)
         byTimebaseReg := 12+32;
       end;

  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;

end; { SetTimebase }


procedure SetCalOutput( var rFreq : Real );
(*
 * DESCRIPTION:
 *   This routine sets the frequency for the square wave generator output
 * INPUT:
 *   rFreq       : the frequency to set
 * OUTPUT:
 *   rFreq       : the set frequency
 *)
const MaxFreq         = 1e5;   { max frequency for cal output }
var   wDivisionFactor : Word;
      rInputFreq      : Real;
      byTimebaseReg   : Byte;

begin

  rInputFreq := 4e6;
 (*
  * clip the input value
  *)
  if   rFreq >  MaxFreq
  then rFreq := MaxFreq;

 (*
  * calculate DivisionFactor
  *)
  wDivisionFactor := round(rInputFreq/rFreq);
  rFreq           := rInputFreq/wDivisionFactor;

 (*
  * initIC 8254, control word A1,A0=11
  *)
  byTimebaseReg := 3+32;

 (*
  * activate timebase register
  *)
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;
  port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
  port[DeviceAddress+DataReg[1]] := 182; { 10 11 011 0 }
     (*
      * 10  [ = select counter 2                   ]
      * 11  [ = write least significant byte first ]
      * 011 [ = mode 3, divide by n counter        ]
      * 0   [ = 16-bits counter                    ]
      *)

 (*
  * load divisionfactor in counter 2
  *)
  byTimebaseReg:=2+32;

 (*
  * activate timebase register
  *)
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;
  port[DeviceAddress+ContReg[1]] := WriteTimebaseCon;
  port[DeviceAddress+DataReg[1]] := lo( wDivisionFactor );
  port[DeviceAddress+DataReg[1]] := hi( wDivisionFactor );

  byTimebaseReg := 3+32;

 (*
  * activate timebase register
  *)
  port[DeviceAddress+ContReg[1]] := TimebaseCon;
  port[DeviceAddress+DataReg[1]] := byTimebaseReg;

end; { SetCalOutput }


procedure ClearSram;
(*
 * DESCRIPTION:
 *   This routine clears the SRAM on the TP208 card
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
const TP208MemSize = 32 * 1024;
      MaxFreq      = 20e+6;
var nCh   : Integer;
    rFreq : Real;
begin

 (*
  * set clck source to PC
  *)
  port[DeviceAddress+ContReg[1]] := 3;
  port[DeviceAddress+DataReg[1]] := 0;
  SetPostCount(TP208MemSize+1);
  for nCh := Ch1 to Ch2 do
  begin
   (*
    * set data/zero on zero
    *)
    port[DeviceAddress+ContReg[nCh+1]] := 4;
    port[DeviceAddress+DataReg[nCh+1]] := 1;
  end;

 (*
  * select fastest timebase
  *)
  rFreq := MaxFreq;
  SetTimebase(rFreq);

 (*
  * set clck source
  *)
  port[DeviceAddress+ContReg[1]]:=3;
  port[DeviceAddress+DataReg[1]]:=14;

 (*
  * reset post-counter and address-counter
  *)
  port[DeviceAddress+ContReg[1]]:=4;
  port[DeviceAddress+DataReg[1]]:=0;

 (*
  * set clck on
  *)
  port[DeviceAddress+ContReg[1]]:=3;
  port[DeviceAddress+DataReg[1]]:=15;

end;{ClearSram}


procedure GotoSramPos( nPos : Integer );
(*
 * DESCRIPTION:
 *   This routine places a 'pointer' to a requested position in the
 *   TP208 SRAM
 * INPUT:
 *   nPos        : the requested position
 * OUTPUT:
 *   -
 *)
const MaxFreq = 20e+6;
var   rFreq : Real;
begin
 (*
  * set clck source to PC
  *)
  port[DeviceAddress+ContReg[1]] := 3;
  port[DeviceAddress+DataReg[1]] := 0;
  SetPostCount(nPos);
  rFreq := MaxFreq;
  SetTimebase(rFreq);

 (*
  * reset postcounter
  *)
  port[DeviceAddress+0]:=4;
  port[DeviceAddress+1]:=0;

 (*
  * set trig on no trig
  *)
  port[DeviceAddress+0]:=7;
  port[DeviceAddress+1]:=2;

 (*
  * set clck source
  *)
  port[DeviceAddress+0]:=3;
  port[DeviceAddress+1]:=2;

 (*
  * set clck on
  *)
  port[DeviceAddress+0]:=3;
  port[DeviceAddress+1]:=3;

  repeat
   (*
    * just do nothing until ready
    *)
  until EOC;

  port[DeviceAddress+0]:=3;
  port[DeviceAddress+1]:=0;

end; { GotoSramPos }


procedure ReadDataSram(     InstrumentSettings : InstrumentSettingsType;
                        var Data               : MeasurementDataType
                      );
(*
 * DESCRIPTION:
 *   This routine reads the data from the SRAM of the TP208 to a data-array
 *   Data
 * INPUT:
 *   InstrumentSettings : some settings of the TP208
 * OUTPUT:
 *   Data               : array containing the measured data
 *)
var nSampleNumber  : Integer;
    lHelp          : Longint;
    wCount         : Word;
begin
 (*
  * set clck source to PC
  *)
  port[DeviceAddress+ContReg[1]] := 3;
  port[DeviceAddress+DataReg[1]] := 0;

 (*
  * read address counter
  *)
  wCount := (256*port[DeviceAddress+1] + port[DeviceAddress+0]) and 32767;
  lHelp  := wCount - InstrumentSettings.lPostSamples - InstrumentSettings.lPreSamples - 6;
  if   lHelp < 0
  then lHelp := lHelp + 32767;
  GotoSramPos( lHelp );

  for nSampleNumber := -InstrumentSettings.lPreSamples
                    to  InstrumentSettings.lPostSamples-1
  do begin
      (*
       * read Data
       *)
       Data[1][nSampleNumber] := port[DeviceAddress+4];
       Data[2][nSampleNumber] := port[DeviceAddress+6];

      (*
       * inc address Counter
       *)
       port[DeviceAddress+ContReg[1]] := 5;
       port[DeviceAddress+DataReg[1]] := 0;
     end;

 (*
  * reset trig channel 1
  *)
  port[DeviceAddress+ContReg[2]] := 5;
  port[DeviceAddress+DataReg[2]] := 0;

 (*
  * reset trig channel 2
  *)
  port[DeviceAddress+ContReg[3]] := 5;
  port[DeviceAddress+DataReg[3]] := 0;

  ClearSram;

end; { ReadDataSram }


procedure SetupForMeasurement( InstrumentSettings : InstrumentSettingsType );
(*
 * DESCRIPTION:
 *   This routine sets up the TP208 to perform a measurement;
 * INPUT:
 *   InstrumentSettings      : a record, containing the settings for the TP208
 * OUTPUT:
 *   -
 *)
begin

  SetTriglevel( InstrumentSettings );
  with InstrumentSettings do
  begin
    SetTimebase ( rSampleFreq );
    SetPostCount( lPostSamples );
  end;

  port[DeviceAddress+ContReg[2]] := 4;
  port[DeviceAddress+DataReg[2]] := 0; {clock channel 1 normal}
  port[DeviceAddress+ContReg[3]] := 4;
  port[DeviceAddress+DataReg[3]] := 0; {clock channel 2 normal}

 (*
  * reset post counter and address counter
  *)
  port[DeviceAddress+ContReg[1]] := 4;
  port[DeviceAddress+DataReg[1]] := 0;

 (*
  * set clck source
  *)
  port[DeviceAddress+ContReg[1]] := 3;
  port[DeviceAddress+DataReg[1]] := 0;

 (*
  * set trigger source
  *)
  port[DeviceAddress+ContReg[1]] := 7;
  case InstrumentSettings.byTrigSource of
    Ch1Trig : port[DeviceAddress+DataReg[1]] := 0;
    ExtTrig : port[DeviceAddress+DataReg[1]] := 1;
    Ch2Trig : port[DeviceAddress+DataReg[1]] := 1;
    NoTrig  : port[DeviceAddress+DataReg[1]] := 2;
    else      port[DeviceAddress+DataReg[1]] := 3;
  end; {case}

 (*
  * reset post counter and address counter
  *)
  port[DeviceAddress+ContReg[1]] := 4;
  port[DeviceAddress+DataReg[1]] := 0;

 (*
  * reset trig channel 1
  *)
  port[DeviceAddress+ContReg[2]] := 5;
  port[DeviceAddress+DataReg[2]] := 0;

 (*
  * reset trig channel 2
  *)
  port[DeviceAddress+ContReg[3]] := 5;
  port[DeviceAddress+DataReg[3]] := 0;

 (*
  * set clck sram
  *)
  port[DeviceAddress+ContReg[1]] :=  3;
  port[DeviceAddress+DataReg[1]] := 15;

end; { SetupForMeasurement }


procedure CalibrateOffset;
(*
 * DESCRIPTION:
 *   This routine calibrates the offset of the TP208
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
const OffsetCount   = 50;
      MaxErrorValue = 10;
      OffsetValue   = 10;
var Switch,
    Index,
    nCh           : Integer;
    HelpValue     : Longint;
    InstrumentSettings : InstrumentSettingsType;
    Data          : MeasurementDataType;
    PpgaSwitch    : array[1..2,1..6] of Longint;
    OffsetAdHelp  : array[1..2] of Real;
begin
  OffsetAdHelp[1] := OffsetAd[1];
  OffsetAdHelp[2] := OffsetAd[2];
  OffsetAd[1]     := -2.0*(OffsetValue/256);
  OffsetAd[2]     := -2.0*(OffsetValue/256);
  PgaSwitch[1][1] := Pga1Gain10 + Pga1Gain4;
  PgaSwitch[1][2] := Pga1Gain10 + Pga1Gain2;
  PgaSwitch[1][3] := Pga1Gain10 + Pga1Gain1;
  PgaSwitch[1][4] := Pga1Gain4;
  PgaSwitch[1][5] := Pga1Gain2;
  PgaSwitch[1][6] := Pga1Gain1;

  PgaSwitch[2][1] := Pga2Gain10 + Pga2Gain4 + Pga2On;
  PgaSwitch[2][2] := Pga2Gain10 + Pga2Gain2 + Pga2On;
  PgaSwitch[2][3] := Pga2Gain10 + Pga2Gain1 + Pga2On;
  PgaSwitch[2][4] := Pga2Gain4  + Pga2On;
  PgaSwitch[2][5] := Pga2Gain2  + Pga2On;
  PgaSwitch[2][6] := Pga2Gain1  + Pga2On;

  for nCh := Ch1 to Ch2 do
  begin
    for Switch := 1 to 6 do
    begin
      SetRef( ValueRef[Ch1][Switch], ValueRef[Ch2][Switch] );
      SetPga( true, true, PgaSwitch[Ch1][Switch] + PgaSwitch[Ch2][Switch] + RefOn );
      repeat
        case nCh of
          Ch1 : begin
                  SetDa( Ch1, ValueDa[Ch1][Switch] + OffsetAd[Ch1] );
                  SetDa( Ch2, 0 );
                end;
          Ch2 : begin
                  SetDa( Ch1, 0 );
                  SetDa( Ch2, ValueDa[Ch2][Switch] + OffsetAd[Ch2] );
                end;
        end; {case}
        with InstrumentSettings do
        begin
          rSampleFreq      := 4e6;
          lPostSamples     := OffsetCount+5;
          lPreSamples      := OffsetCount;
          byTrigHys        := 0;
          byTrigLevel[Ch1] := 128;
          byTrigLevel[Ch2] := 128;
          bTrigSlope       := PosSlope;
          byTrigSource     := NoTrig;
        end;
        SetupForMeasurement( InstrumentSettings );

{!!}    index := 0;

        repeat
         (*
          * just do nothing
          *)
        until Eoc;
        ReadDataSram( InstrumentSettings, Data );

        HelpValue:=0;
        for Index:=1 to OffsetCount do
          HelpValue := HelpValue+Data[nCh][Index];

        if   HelpValue > (OffsetValue*OffsetCount)+MaxErrorValue
        then ValueDa[nCh][Switch] := ValueDa[nCh][Switch]+5/2048;

        if   HelpValue<(OffsetValue*OffsetCount)-MaxErrorValue
        then ValueDa[nCh][Switch] := ValueDa[nCh][Switch]-5/2048;

      until (  (HelpValue<=(OffsetValue*OffsetCount)+MaxErrorValue) and
               (HelpValue>=(OffsetValue*OffsetCount)-MaxErrorValue)
            );
    end;
  end;

  for nCh := Ch1 to Ch2 do
  begin
    ValueDa[nCh][ 7]:=ValueDa[nCh][4];
    ValueDa[nCh][ 8]:=ValueDa[nCh][5];
    ValueDa[nCh][ 9]:=ValueDa[nCh][6];
    ValueDa[nCh][10]:=ValueDa[nCh][4];
    ValueDa[nCh][11]:=ValueDa[nCh][5];
    ValueDa[nCh][12]:=ValueDa[nCh][6];
  end;

  OffsetAd[Ch1]:=OffsetAdHelp[Ch1];
  OffsetAd[Ch2]:=OffsetAdHelp[Ch2]

end; { CalibrateOffset }


procedure CalibrateGain;
(*
 * DESCRIPTION:
 *   This routine calibrates the TP208 gain
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
const GainNumber    = 100;
      MaxErrorValue = 20;
      MaxValue      = 250/256;
var HelpValue     : Longint;
    InstrumentSettings : InstrumentSettingsType;
    Data          : MeasurementDataType;
    I,J,
    meet_waarde   : Longint;
    PgaSwitch     : array[1..2,1..6] of Longint;
    middel,
    ChNumber,
    Switch        : Integer;
    CalValue      : Real;
begin
  PgaSwitch[1][1] := Pga1Gain10 + Pga1Gain4;
  PgaSwitch[1][2] := Pga1Gain10 + Pga1Gain2;
  PgaSwitch[1][3] := Pga1Gain10 + Pga1Gain1;
  PgaSwitch[1][4] := Pga1Gain4;
  PgaSwitch[1][5] := Pga1Gain2;
  PgaSwitch[1][6] := Pga1Gain1;

  PgaSwitch[2][1] := Pga2Gain10 + Pga2Gain4 + Pga2On;
  PgaSwitch[2][2] := Pga2Gain10 + Pga2Gain2 + Pga2On;
  PgaSwitch[2][3] := Pga2Gain10 + Pga2Gain1 + Pga2On;
  PgaSwitch[2][4] := Pga2Gain4  + Pga2On;
  PgaSwitch[2][5] := Pga2Gain2  + Pga2On;
  PgaSwitch[2][6] := Pga2Gain1  + Pga2On;

  for Switch := 1 to 6 do
  begin
    CalValue := PgaSwitchValue[Switch]*MaxValue*25;
    if   CalValue >  4.96093
    then CalValue := 4.96093;
    meet_waarde   := round( (128+(128*0.04*CalValue/PgaSwitchValue[Switch])) *
                            GainNumber
                          );
    SetDa ( 1, OffsetAd[1] + ValueDa[1][Switch]);
    SetDa ( 2, CalValue);
    SetPga( true, true, PgaSwitch[1][Switch]+PgaSwitch[2][Switch]+RefOn );
    middel := 0;
    repeat
      inc(middel);
      SetRef(ValueRef[1][Switch], ValueRef[2][Switch]);
      delay(10);
      with InstrumentSettings do
      begin
        rSampleFreq      := 4e6;
        lPostSamples     := GainNumber+600;
        lPreSamples      := GainNumber;
        byTrigHys        := 0;
        byTrigLevel[Ch1] := 128;
        byTrigLevel[Ch2] := 128;
        bTrigSlope       := PosSlope;
        byTrigSource     := NoTrig;
      end;
      SetupForMeasurement( InstrumentSettings );
      J:=0;
      repeat
        for I := 1 to 10 do
          inc(J);
      until Eoc;
      ReadDataSram(InstrumentSettings,Data);

      HelpValue:=0;
      for I := 1+500 to GainNumber+500 do
        HelpValue:=HelpValue+Data[1][I];
      if   HelpValue > meet_waarde+MaxErrorValue
      then ValueRef[1][Switch] := ValueRef[1][Switch] + 1;
      if   HelpValue < meet_waarde-MaxErrorValue
      then ValueRef[1][Switch] := ValueRef[1][Switch] - 1;

    until (  (HelpValue <= meet_waarde+MaxErrorValue) and
             (HelpValue>=meet_waarde-MaxErrorValue)
          )
          or ( (ValueRef[1][Switch]>255) or (ValueRef[1][Switch]<1) )
          or (middel>128);
  end;


  for Switch:=1 to 6 do
  begin
    CalValue := PgaSwitchValue[Switch]*MaxValue*25;
    if   CalValue >  4.96093
    then CalValue := 4.96093;
    meet_waarde := round( (2048/16+(2048/16*0.04*CalValue/PgaSwitchValue[Switch])) *
                          GainNumber
                        );
    SetDa(1,CalValue);
    SetDa(2,OffsetAd[2]+ValueDa[2][Switch]);
    SetPga(true,true,PgaSwitch[1][Switch]+PgaSwitch[2][Switch]+RefOn);
    middel:=0;
    repeat
      inc(middel);
      SetRef(ValueRef[1][Switch], ValueRef[2][Switch]);
      delay(10);
      with InstrumentSettings do
      begin
        rSampleFreq  := 4e6;
        lPostSamples := GainNumber+600;
        lPreSamples  := GainNumber;
        byTrigHys     := 0;
        bTrigSlope   := PosSlope;
        byTrigSource  := NoTrig;
      end;
      SetupForMeasurement( InstrumentSettings );

      J:=0;
      repeat
        for I:=1 to 10 do
          inc(J);
      until EOC;
      ReadDataSram(InstrumentSettings,Data);

      HelpValue:=0;
      for I:=1+500 to GainNumber+500 do
        HelpValue := HelpValue+Data[2][I];
      if   HelpValue > meet_waarde+MaxErrorValue
      then ValueRef[2][Switch] := ValueRef[2][Switch] + 1;;
      if   HelpValue < meet_waarde-MaxErrorValue
      then ValueRef[2][Switch] := ValueRef[2][Switch] - 1;
    until ((HelpValue<=meet_waarde+MaxErrorValue) and (HelpValue>=meet_waarde-MaxErrorValue))
       or ((ValueRef[2][Switch]>255) or (ValueRef[2][Switch]<1))
       or (middel>128);
  end;

  for ChNumber:=1 to 2 do
  begin
    ValueRef[ChNumber][ 7] := ValueRef[ChNumber][4];
    ValueRef[ChNumber][ 8] := ValueRef[ChNumber][5];
    ValueRef[ChNumber][ 9] := ValueRef[ChNumber][6];
    ValueRef[ChNumber][10] := ValueRef[ChNumber][4];
    ValueRef[ChNumber][11] := ValueRef[ChNumber][5];
    ValueRef[ChNumber][12] := ValueRef[ChNumber][6];
  end;

end; { CalibrateGain }


procedure CalibrateInstrument( var CalError : Boolean );
(*
 * DESCRIPTION:
 *   This routine calibrates the TP208
 * INPUT:
 *   -
 * OUTPUT:
 *   CalError   : indicates whether an error occurred during calibration
 *)
var I         : Longint;
    ChNumber,
    Switch    : Integer;
begin
  SetDa(1, ValueDa[1][1]+OffsetAd[1]);
  SetDa(2, ValueDa[2][1]+OffsetAd[2]);
  for I := 1 to 2 do
  begin
    CalibrateOffset;
    CalibrateGain;
  end;
  CalibrateOffset;
  CalError := false;
  for ChNumber := 1 to 2 do
    for Switch := 1 to 12 do
      if   (ValueRef[ChNumber][Switch]<10) or (ValueRef[ChNumber][Switch]>250)
      then CalError := true;
  if CalError then
  begin
    for I := 1 to 2 do
    begin
      CalibrateOffset;
      CalibrateGain;
    end;
    CalibrateOffset;
  end;
  CalError := false;
  for ChNumber := 1 to 2 do
    for Switch := 1 to 12 do
      if   (ValueRef[ChNumber][Switch]<10) or (ValueRef[ChNumber][Switch]>250)
      then CalError := true;
  ValueDa[1][1] := ValueDa[1][1]-40/2048;
  ValueDa[2][1] := ValueDa[2][1]-40/2048;
  ValueDa[1][2] := ValueDa[1][2]-10/2048;
  ValueDa[2][2] := ValueDa[2][2]-10/2048;

end; { CalibrateInstrument }


function HardwarePresent : Boolean;
(*
 * DESCRIPTION:
 *   This routine determines whether a TP208 card is present in the PC
 * INPUT:
 *   -
 * OUTPUT:
 *   returnvalue        : True if a card is detected
 *                        False if no card is detected
 *)
var Index  : Integer;
    nValue : Integer;
begin

  for Index := 1 to 10 do
    nValue := port[DeviceAddress+3];
  if   nValue = 0
  then HardwarePresent := true
  else HardwarePresent := false;

end; { HardwarePresent }


procedure InitInstrument;
(*
 * DESCRIPTION:
 *   This routine initializes the PC to use the TP208 card and
 *   the TP208 card to measure with
 * INPUT:
 *   -
 * OUTPUT:
 *   -
 *)
var nCh,
    nSwitch   : Integer;
    bCalError : Boolean;
begin
 (*
  * save the original Timer interrupt vector of the PC
  * and assign a new interrupt routine to the vector
  *)
  getintvec( $1c,  Int1cSave    );
  setintvec( $1c, @TimeOutTimer );

  ExitSave := ExitProc;
  ExitProc := @ProgramExit;

  DeviceAddress := TP208Address;

 (*
  * check whether a TP208 is present
  *)
  if not HardwarePresent then
  begin
    write( strText2 );
    exit;
  end;

 (*
  * if this point is achieved, a TP208 is present.
  *
  * set it up
  *)

  OffsetAd[Ch1] := -1.0;
  OffsetAd[Ch2] := -1.0;

  for nCh := Ch1 to Ch2 do
    for nSwitch := Pga20mVolt to Pga80Volt do
      ValueDa[nCh][nSwitch] := 0.0;

  for nCh := Ch1 to Ch2 do
    for nSwitch := Pga20mVolt to Pga80Volt do
      ValueRef[nCh][nSwitch] := 127;

  PgaSwitchValue[ 1] :=  20e-3;
  PgaSwitchValue[ 2] :=  40e-3;
  PgaSwitchValue[ 3] :=  80e-3;
  PgaSwitchValue[ 4] := 200e-3;
  PgaSwitchValue[ 5] := 400e-3;
  PgaSwitchValue[ 6] := 800e-3;
  PgaSwitchValue[ 7] :=   2;
  PgaSwitchValue[ 8] :=   4;
  PgaSwitchValue[ 9] :=   8;
  PgaSwitchValue[10] :=  20;
  PgaSwitchValue[11] :=  40;
  PgaSwitchValue[12] :=  80;



 (*
  * calibrate the T208
  *)
  calibrateInstrument(bCalError);
  if bCalError then
  begin
    write(strText1);
    exit;
  end;


  PgaSwitch[1][ 1] := Pga1Gain4 + Pga1Att1  + Pga1Gain10;
  PgaSwitch[1][ 2] := Pga1Gain2 + Pga1Att1  + Pga1Gain10;
  PgaSwitch[1][ 3] := Pga1Gain1 + Pga1Att1  + Pga1Gain10;
  PgaSwitch[1][ 4] := Pga1Gain4 + Pga1Att1  ;
  PgaSwitch[1][ 5] := Pga1Gain2 + Pga1Att1  ;
  PgaSwitch[1][ 6] := Pga1Gain1 + Pga1Att1  ;
  PgaSwitch[1][ 7] := Pga1Gain4 + Pga1Att10 ;
  PgaSwitch[1][ 8] := Pga1Gain2 + Pga1Att10 ;
  PgaSwitch[1][ 9] := Pga1Gain1 + Pga1Att10 ;
  PgaSwitch[1][10] := Pga1Gain4 + Pga1Att100;
  PgaSwitch[1][11] := Pga1Gain2 + Pga1Att100;
  PgaSwitch[1][12] := Pga1Gain1 + Pga1Att100;

  PgaSwitch[2][ 1] := Pga2Gain4 + Pga2Att1   + Pga2Gain10 + Pga2on;
  PgaSwitch[2][ 2] := Pga2Gain2 + Pga2Att1   + Pga2Gain10 + Pga2on;
  PgaSwitch[2][ 3] := Pga2Gain1 + Pga2Att1   + Pga2Gain10 + Pga2on;
  PgaSwitch[2][ 4] := Pga2Gain4 + Pga2Att1   +              Pga2on;
  PgaSwitch[2][ 5] := Pga2Gain2 + Pga2Att1   +              Pga2on;
  PgaSwitch[2][ 6] := Pga2Gain1 + Pga2Att1   +              Pga2on;
  PgaSwitch[2][ 7] := Pga2Gain4 + Pga2Att10  +              Pga2on;
  PgaSwitch[2][ 8] := Pga2Gain2 + Pga2Att10  +              Pga2on;
  PgaSwitch[2][ 9] := Pga2Gain1 + Pga2Att10  +              Pga2on;
  PgaSwitch[2][10] := Pga2Gain4 + Pga2Att100 +              Pga2on;
  PgaSwitch[2][11] := Pga2Gain2 + Pga2Att100 +              Pga2on;
  PgaSwitch[2][12] := Pga2Gain1 + Pga2Att100 +              Pga2on;

  ClearSram;

end; { InitInstrument }


procedure StartMeasurement(  var MeasureSetting : MeasurementSettingsType;
                             var MeasureData    : MeasurementDataType
                           );
(*
 * DESCRIPTION:
 *   This routine performs 1 measurement with the TP208
 * INPUT:
 *   MeasureSetting : a structure holding the measurement settings
 * OUTPUT:
 *   MeasureData    : an array holding the measured data
 *)
var InstrumentSetting   : InstrumentSettingsType;
    lSwitchSetting      : Longint;
begin
  with MeasureSetting do
  begin
    lSwitchSetting := PgaSwitch[1][bySwitch[1]] + PgaSwitch[2][bySwitch[2]];

    SetDa(Ch1, rDcLevel[1] + OffsetAd[1] + ValueDa[1][bySwitch[1]]);
    SetDa(Ch2, rDcLevel[2] + OffsetAd[2] + ValueDa[2][bySwitch[2]]);
    SetRef(ValueRef[1][bySwitch[1]],ValueRef[2][bySwitch[2]]);
    SetPga(bCoupling[Ch1],bCoupling[Ch2],lSwitchSetting);

    InstrumentSetting.rSampleFreq      := rSampleFreq;
    InstrumentSetting.lPostSamples     := lPostSamples;
    InstrumentSetting.lPreSamples      := lPreSamples;
    InstrumentSetting.byTrigHys        := byTrigHys;
    InstrumentSetting.byTrigLevel[Ch1] := byTrigLevel[Ch1];
    InstrumentSetting.byTrigLevel[Ch2] := byTrigLevel[Ch2];
    InstrumentSetting.bTrigSlope       := bTrigSlope;
    InstrumentSetting.byTrigSource     := nTrigSource;
  end;
  SetupForMeasurement(InstrumentSetting);

  asm
    CLI             { disable interrupts    }
  end;
  lTimerCount := 0; { clear timeout counter }
  asm
    STI             { enable interrupts     }
  end;

  repeat
   (*
    * This loop is to wait for the End of Conversion signal
    * In this loop, the timeout counter is monitored.
    * On interruptbase this counter is increased 18.2 times per second.
    * If the timeout value is reached, a start of measurement is forced
    * by setting the Trigger bit.
    * Waiting continues until an End Of Conversion is detected
    *)
    if   (lTimerCount >= MeasureSetting.lTimeOut)
    then SetTrigbit;
  until EOC;

  if EOC then
  begin
    ReadDataSram( InstrumentSetting, MeasureData );
  end;

end; { StartMeasurement }